# sage_setup: distribution = sagemath-repl
# distutils: language = c++
"""
Example of a class wrapping an STL vector

EXAMPLES::

    sage: from sage.tests.stl_vector import stl_int_vector
    sage: v = stl_int_vector()
    sage: v
    A vector of integers
    vector<int>:
     data[0] = 123
     data[1] = 456

AUTHORS:

- Volker Braun (2012-01-18): initial version
"""
# ****************************************************************************
#       Copyright (C) 2012 Volker Braun <vbraun.name@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  http://www.gnu.org/licenses/
# ****************************************************************************

from cysignals.signals cimport sig_on, sig_off

from sage.structure.sage_object cimport SageObject
from sage.rings.integer cimport Integer
from sage.libs.gmp.mpz cimport mpz_add_ui
from libcpp.vector cimport vector
from libcpp.string cimport string
from sage.structure.richcmp cimport richcmp_not_equal, rich_to_bool
from sage.cpython.string cimport char_to_str


cdef class stl_int_vector(SageObject):
    """
    Example class wrapping an STL vector.

    EXAMPLES::

        sage: from sage.tests.stl_vector import stl_int_vector
        sage: v = stl_int_vector()
    """

    cdef vector[int] *data
    cdef string *name

    def __cinit__(self):
        """
        The Cython constructor.

        EXAMPLES::

            sage: from sage.tests.stl_vector import stl_int_vector
            sage: v = stl_int_vector()   # indirect doctest
            sage: TestSuite(v)
            Test suite for A vector of integers
            vector<int>:
             data[0] = 123
             data[1] = 456
        """
        self.data = new vector[int]()
        self.name = new string(<char*>"A vector of integers\n")
        self.data.push_back(123)
        self.data.push_back(456)

    def __dealloc__(self):
        """
        The Cython destructor.

        EXAMPLES::

            sage: from sage.tests.stl_vector import stl_int_vector
            sage: v = stl_int_vector()   # indirect doctest
        """
        del self.data
        del self.name

    def __getitem__(self, int i):
        """
        Return the ``i``-th element.

        EXAMPLES::

            sage: from sage.tests.stl_vector import stl_int_vector
            sage: v = stl_int_vector()
            sage: v[1]
            456
        """
        assert 0 <= i < self.data.size()
        return self.data.at(i)

    def __repr__(self):
        """
        Return a string representation.

        EXAMPLES::

            sage: from sage.tests.stl_vector import stl_int_vector
            sage: v = stl_int_vector()
            sage: v
            A vector of integers
            vector<int>:
             data[0] = 123
             data[1] = 456
        """
        s = char_to_str(self.name.c_str())
        s += 'vector<int>:\n'
        for i in range(self.data.size()):
            s += ' data[' + str(i) + '] = ' + str(self.data.at(i)) + '\n'
        return s.strip()

    cpdef sum(self):
        """
        Add the elements.

        EXAMPLES::

            sage: from sage.tests.stl_vector import stl_int_vector
            sage: v = stl_int_vector()
            sage: v.sum()
            579
        """
        cdef Integer accumulator = Integer(0)
        cdef vector[int].iterator i = self.data.begin()
        sig_on()
        while i != self.data.end():
            mpz_add_ui(accumulator.value, accumulator.value, <int>(i[0]))
            i += 1
        sig_off()
        return accumulator

    def __richcmp__(left, stl_int_vector right, int op):
        """
        Compare with ``other``.

        EXAMPLES::

            sage: from sage.tests.stl_vector import stl_int_vector
            sage: u = stl_int_vector()
            sage: v = stl_int_vector()
            sage: u == v
            True
        """
        cdef stl_int_vector lhs = left
        cdef stl_int_vector rhs = right

        lx = lhs.data.size()
        rx = rhs.data.size()
        if lx != rx:
            return richcmp_not_equal(lx, rx, op)

        cdef vector[int].iterator lhs_iter = lhs.data.begin()
        cdef vector[int].iterator rhs_iter = rhs.data.begin()
        sig_on()
        try:
            while lhs_iter != lhs.data.end():
                left_i = <int>(lhs_iter[0])
                right_i = <int>(rhs_iter[0])
                if left_i != right_i:
                    return richcmp_not_equal(left_i, right_i, op)
                lhs_iter += 1
                rhs_iter += 1
        finally:
            sig_off()
        return rich_to_bool(op, 0)



# sage_setup: distribution = sagemath-repl
##          Sage Doctest File         ##
#**************************************#
#*    Generated from PreTeXt source   *#
#*    on 2017-08-24T11:43:34-07:00    *#
#*                                    *#
#*   http://mathbook.pugetsound.edu   *#
#*                                    *#
#**************************************#
##
"""
Please contact Rob Beezer (beezer@ups.edu) with
any test failures here that need to be changed
as a result of changes accepted into Sage.  You
may edit/change this file in any sensible way, so
that development work may procede.  Your changes
may later be replaced by the authors of "Abstract
Algebra: Theory and Applications" when the text is
updated, and a replacement of this file is proposed
for review.
"""
##
## To execute doctests in these files, run
##   $ $SAGE_ROOT/sage -t <directory-of-these-files>
## or
##   $ $SAGE_ROOT/sage -t <a-single-file>
##
## Replace -t by "-tp n" for parallel testing,
##   "-tp 0" will use a sensible number of threads
##
## See: http://www.sagemath.org/doc/developer/doctesting.html
##   or run  $ $SAGE_ROOT/sage --advanced  for brief help
##
## Generated at 2017-08-24T11:43:34-07:00
## From "Abstract Algebra"
## At commit 26d3cac0b4047f4b8d6f737542be455606e2c4b4
##
## Section 7.6 Sage
##
r"""
~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: p_a = next_prime(10^10)
    sage: q_a = next_prime(p_a)
    sage: p_b = next_prime((3/2)*10^10)
    sage: q_b = next_prime(p_b)
    sage: n_a = p_a * q_a
    sage: n_b = p_b * q_b
    sage: n_a, n_b
    (100000000520000000627, 225000000300000000091)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: m_a = euler_phi(n_a)
    sage: m_b = euler_phi(n_b)
    sage: m_a, m_b
    (100000000500000000576, 225000000270000000072)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: factor(m_a)
    2^6 * 3 * 11 * 17 * 131 * 521 * 73259 * 557041

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: E_a = 5*23
    sage: D_a = inverse_mod(E_a, m_a)
    sage: D_a
    20869565321739130555

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: factor(m_b)
    2^3 * 3^4 * 107 * 1298027 * 2500000001

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: E_b = 7*29
    sage: D_b = inverse_mod(E_b, m_b)
    sage: D_b
    24384236482463054195

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: print("Alice's public key, n:", n_a, "E:", E_a)
    Alice's public key, n: 100000000520000000627 E: 115

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: print("Alice's private key, D:", D_a)
    Alice's private key, D: 20869565321739130555

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: print("Bob's public key, n:", n_b, "E:", E_b)
    Bob's public key, n: 225000000300000000091 E: 203

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: print("Bob's private key, D:", D_b)
    Bob's private key, D: 24384236482463054195

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: word = 'Sage'
    sage: digits = [ord(letter) for letter in word]
    sage: digits
    [83, 97, 103, 101]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: message = ZZ(digits, 128)
    sage: message
    213512403

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: signed = power_mod(message, D_a, n_a)
    sage: signed
    47838774644892618423

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: encrypted = power_mod(signed, E_b, n_b)
    sage: encrypted
    111866209291209840488

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: decrypted = power_mod(encrypted, D_b, n_b)
    sage: decrypted
    47838774644892618423

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: received = power_mod(decrypted, E_a, n_a)
    sage: received
    213512403

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: digits = received.digits(base=128)
    sage: letters = [chr(ascii) for ascii in digits]
    sage: letters
    ['S', 'a', 'g', 'e']

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: ''.join(letters)
    'Sage'
"""



# sage_setup: distribution = sagemath-repl
"""
This file (./sol/linsolve_doctest.sage) was *autogenerated* from ./sol/linsolve.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./sol/linsolve_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./sol/linsolve.tex, line 72::

  sage: m = random_matrix(RDF,4)
  sage: a = transpose(m)*m
  sage: c = a.cholesky()
  sage: U,S,V = c.SVD()
  sage: X = U*S*transpose(U)

Sage example in ./sol/linsolve.tex, line 84::

  sage: M = (X*X-a)
  sage: all(abs(M[i,j]) < 10^-13
  ....:     for i in range(4) for j in range(4) )
  True
"""



# sage_setup: distribution = sagemath-repl
import argparse
import os
import sys

# Note: the DOT_SAGE and SAGE_STARTUP_FILE environment variables have already been set by sage-env
DOT_SAGE = os.environ.get('DOT_SAGE', os.path.join(os.environ.get('HOME'),
                                                   '.sage'))

# Override to not pick up user configuration, see Issue #20270
os.environ['SAGE_STARTUP_FILE'] = os.path.join(DOT_SAGE, 'init-doctests.sage')


def _get_optional_defaults():
    """Return the default value for the --optional flag."""
    optional = ['sage', 'optional']

    return ','.join(optional)


def _make_parser():
    r"""
    Return the :class:`argparse.ArgumentParser`.

    TESTS:

    Test that the defaults are the consistent::

        sage: from sage.doctest.control import DocTestDefaults
        sage: from sage.doctest.__main__ import _make_parser
        sage: os.environ.pop('SAGE_DOCTEST_RANDOM_SEED', None)
        ...
        sage: parser = _make_parser()
        sage: args = parser.parse_args([])
        sage: DD = DocTestDefaults(runtest_default=True); DD
        DocTestDefaults(abspath=False, file_iterations=0, global_iterations=0,
                        optional='sage,optional', random_seed=None,
                        stats_path='.../timings2.json')
        sage: D = copy(args.__dict__)
        sage: del D['filenames']
        sage: DA = DocTestDefaults(runtest_default=True, **D); DA
        DocTestDefaults(abspath=False, file_iterations=0, global_iterations=0,
                        optional='sage,optional', random_seed=None,
                        stats_path='.../timings2.json')
    """
    parser = argparse.ArgumentParser(usage="sage -t [options] filenames",
                                     description="Run all tests in a file or a list of files whose extensions "
                                                 "are one of the following: "
                                                 ".py, .pyx, .pxd, .pxi, .sage, .spyx, .tex, .rst.")
    parser.add_argument("-p", "--nthreads", dest="nthreads",
                        type=int, nargs='?', const=0, default=1, metavar="N",
                        help="test in parallel using N threads, with 0 interpreted as max(2, min(8, cpu_count())); "
                        "when run under the control of the GNU make jobserver (make -j), request as most N job slots")
    parser.add_argument("-T", "--timeout", type=int, default=-1, help="timeout (in seconds) for doctesting one file, 0 for no timeout")
    what = parser.add_mutually_exclusive_group()
    what.add_argument("-a", "--all", action="store_true", default=False, help="test all files in the Sage library")
    what.add_argument("--installed", action="store_true", default=False, help="test all installed modules of the Sage library")
    parser.add_argument("--logfile", type=argparse.FileType('a'), metavar="FILE", help="log all output to FILE")

    parser.add_argument("--format", choices=["sage", "github"], default="sage",
                        help="set format of error messages and warnings")
    parser.add_argument("-l", "--long", action="store_true", default=False, help="include lines with the phrase 'long time'")
    parser.add_argument("-s", "--short", dest="target_walltime", nargs='?',
                        type=int, default=-1, const=300, metavar="SECONDS",
                        help="run as many doctests as possible in about 300 seconds (or the number of seconds given as an optional argument)")
    parser.add_argument("--warn-long", dest="warn_long", nargs='?',
                        type=float, default=-1.0, const=1.0, metavar="SECONDS",
                        help="warn if tests take more time than SECONDS")
    # By default, include all tests marked 'sagemath_doc_html' -- see
    # https://github.com/sagemath/sage/issues/25345 and
    # https://github.com/sagemath/sage/issues/26110:
    parser.add_argument("--optional", metavar="FEATURES", default=_get_optional_defaults(),
        help='only run tests including one of the "# optional" tags listed in FEATURES (separated by commas); '
             'if "sage" is listed, will also run the standard doctests; '
             'if "sagemath_doc_html" is listed, will also run the tests relying on the HTML documentation; '
             'if "optional" is listed, will also run tests for installed optional packages or detected features; '
             'if "external" is listed, will also run tests for available external software; '
             'if set to "all", then all tests will be run; '
             'use "!FEATURE" to disable tests marked "# optional - FEATURE". '
             'Note that "!" needs to be quoted or escaped in the shell.')
    parser.add_argument("--hide", metavar="FEATURES", default="",
        help='run tests pretending that the software listed in FEATURES (separated by commas) is not installed; '
             'if "all" is listed, will also hide features corresponding to all optional or experimental packages; '
             'if "optional" is listed, will also hide features corresponding to optional packages.')
    parser.add_argument("--probe", metavar="FEATURES", default="",
        help='run tests that would not be run because one of the given FEATURES (separated by commas) is not installed; '
             'report the tests that pass nevertheless')
    parser.add_argument("--randorder", type=int, metavar="SEED", help="randomize order of tests")
    parser.add_argument("--random-seed", dest="random_seed", type=int, metavar="SEED", help="random seed (integer) for fuzzing doctests",
                        default=os.environ.get("SAGE_DOCTEST_RANDOM_SEED"))
    parser.add_argument("--global-iterations", "--global_iterations", type=int, default=0, help="repeat the whole testing process this many times")
    parser.add_argument("--file-iterations", "--file_iterations", type=int, default=0, help="repeat each file this many times, stopping on the first failure")
    parser.add_argument("--environment", type=str, default="sage.repl.ipython_kernel.all_jupyter", help="name of a module that provides the global environment for tests")

    parser.add_argument("-i", "--initial", action="store_true", default=False, help="only show the first failure in each file")
    parser.add_argument("--exitfirst", action="store_true", default=False, help="end the test run immediately after the first failure or unexpected exception")
    parser.add_argument("--force_lib", "--force-lib", action="store_true", default=False, help="do not import anything from the tested file(s)")
    parser.add_argument("--if-installed", action="store_true", default=False, help="skip Python/Cython files that are not installed as modules")
    parser.add_argument("--abspath", action="store_true", default=False, help="print absolute paths rather than relative paths")
    parser.add_argument("--verbose", action="store_true", default=False, help="print debugging output during the test")
    parser.add_argument("-d", "--debug", action="store_true", default=False, help="drop into a python debugger when an unexpected error is raised")
    parser.add_argument("--only-errors", action="store_true", default=False, help="only output failures, not test successes")

    parser.add_argument("--gdb", action="store_true", default=False, help="run doctests under the control of gdb")
    parser.add_argument("--lldb", action="store_true", default=False, help="run doctests under the control of lldb")
    parser.add_argument("--valgrind", "--memcheck", action="store_true", default=False,
                        help="run doctests using Valgrind's memcheck tool.  The log "
                        "files are named sage-memcheck.PID and can be found in " +
                        os.path.join(DOT_SAGE, "valgrind"))
    parser.add_argument("--massif", action="store_true", default=False,
                        help="run doctests using Valgrind's massif tool.  The log "
                        "files are named sage-massif.PID and can be found in " +
                        os.path.join(DOT_SAGE, "valgrind"))
    parser.add_argument("--cachegrind", action="store_true", default=False,
                        help="run doctests using Valgrind's cachegrind tool.  The log "
                        "files are named sage-cachegrind.PID and can be found in " +
                        os.path.join(DOT_SAGE, "valgrind"))
    parser.add_argument("--omega", action="store_true", default=False,
                        help="run doctests using Valgrind's omega tool.  The log "
                        "files are named sage-omega.PID and can be found in " +
                        os.path.join(DOT_SAGE, "valgrind"))

    parser.add_argument("-f", "--failed", action="store_true", default=False,
        help="doctest only those files that failed in the previous run")
    what.add_argument("-n", "--new", action="store_true", default=False,
        help="doctest only those files that have been changed in the repository and not yet been committed")
    parser.add_argument("--show-skipped", "--show_skipped", action="store_true", default=False,
        help="print a summary at the end of each file of optional tests that were skipped")

    parser.add_argument("--stats_path", "--stats-path", default=os.path.join(DOT_SAGE, "timings2.json"),
                        help="path to a json dictionary for timings and failure status for each file from previous runs; it will be updated in this run")
    parser.add_argument("--baseline_stats_path", "--baseline-stats-path", default=None,
                        help="path to a json dictionary for timings and failure status for each file, to be used as a baseline; it will not be updated")

    class GCAction(argparse.Action):
        def __call__(self, parser, namespace, values, option_string=None):
            gcopts = dict(DEFAULT=0, ALWAYS=1, NEVER=-1)
            new_value = gcopts[values]
            setattr(namespace, self.dest, new_value)

    parser.add_argument("--gc",
                        choices=["DEFAULT", "ALWAYS", "NEVER"],
                        default=0,
                        action=GCAction,
                        help="control garbarge collection "
                        "(ALWAYS: collect garbage before every test; NEVER: disable gc; DEFAULT: Python default)")

    # The --serial option is only really for internal use, better not
    # document it.
    parser.add_argument("--serial", action="store_true", default=False, help=argparse.SUPPRESS)
    # Same for --die_timeout
    parser.add_argument("--die_timeout", type=int, default=-1, help=argparse.SUPPRESS)

    parser.add_argument("filenames", help="file names", nargs='*')
    return parser


def main():
    parser = _make_parser()
    # custom treatment to separate properly
    # one or several file names at the end
    new_arguments = []
    need_filenames = True
    in_filenames = False
    afterlog = False
    for arg in sys.argv[1:]:
        if arg in ('-n', '--new', '-a', '--all', '--installed'):
            need_filenames = False
        elif need_filenames and not (afterlog or in_filenames) and os.path.exists(arg):
            in_filenames = True
            new_arguments.append('--')
        new_arguments.append(arg)
        afterlog = arg in ['--logfile', '--stats_path', '--stats-path',
                           '--baseline_stats_path', '--baseline-stats-path']

    args = parser.parse_args(new_arguments)

    if not args.filenames and not (args.all or args.new or args.installed):
        print('either use --new, --all, --installed, or some filenames')
        return 2

    # Limit the number of threads to 2 to save system resources.
    # See Issue #23713, #23892, #30351
    if sys.platform == 'darwin':
        os.environ["OMP_NUM_THREADS"] = "1"
    else:
        os.environ["OMP_NUM_THREADS"] = "2"

    os.environ["SAGE_NUM_THREADS"] = "2"

    from sage.doctest.control import DocTestController
    DC = DocTestController(args, args.filenames)
    err = DC.run()

    # Issue #33521: Do not run pytest if the pytest configuration is not available.
    # This happens when the source tree is not available and SAGE_SRC falls back
    # to SAGE_LIB.
    from sage.env import SAGE_SRC
    if not all(os.path.isfile(os.path.join(SAGE_SRC, f))
               for f in ["conftest.py", "tox.ini"]):
        return err

    try:
        exit_code_pytest = 0
        import pytest
        pytest_options = []
        if args.verbose:
            pytest_options.append("-v")

        # #35999: no filename in arguments defaults to "src"
        if not args.filenames:
            filenames = [SAGE_SRC]
        else:
            # #31924: Do not run pytest on individual Python files unless
            # they match the pytest file pattern.  However, pass names
            # of directories. We use 'not os.path.isfile(f)' for this so that
            # we do not silently hide typos.
            filenames = [f for f in args.filenames
                         if f.endswith("_test.py") or not os.path.isfile(f)]
        if filenames:
            print(f"Running pytest on {filenames} with options {pytest_options}")
            exit_code_pytest = pytest.main(filenames + pytest_options)
            if exit_code_pytest == 5:
                # Exit code 5 means there were no test files, pass in this case
                exit_code_pytest = 0

    except ModuleNotFoundError:
        print("pytest is not installed in the venv, skip checking tests that rely on it")

    if err == 0:
        return exit_code_pytest
    return err


if __name__ == "__main__":
    sys.exit(main())



# sage_setup: distribution = sagemath-repl
"""
This file (./sol/float_doctest.sage) was *autogenerated* from ./sol/float.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./sol/float_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./sol/float.tex, line 17::

  sage: R100=RealField(100)
  sage: x=R100(10^30)
  sage: x>2^99
  True
  sage: x<2^100
  True

Sage example in ./sol/float.tex, line 48::

  sage: e=2^100
  sage: s1=10^30
  sage: significand=[]
  sage: nbdigits=0 # number of significant digits
  sage: while s1>0:
  ....:    e/=2
  ....:    if e<=s1:
  ....:        significand.append(1)
  ....:        s1-=e
  ....:    else:
  ....:        significand.append(0)
  ....:    nbdigits+=1
  sage: print(significand)
  [1, 1, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 1, 1, 0, 0,
   1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0,
   0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 1,
   1, 1, 1, 0, 1, 0, 1, 0, 0, 1]
  sage: print("number of significant digits: " + str(nbdigits))
  number of significant digits: 70

Sage example in ./sol/float.tex, line 86::

  sage: R100=RealField(100)
  sage: x=R100(10^30)
  sage: s,m,e = x.sign_mantissa_exponent()
  sage: s,m,e
  (1, 1000000000000000000000000000000, 0)

Sage example in ./sol/float.tex, line 126::

  sage: var("u0 u1 u2 alpha beta gamma n")
  (u0, u1, u2, alpha, beta, gamma, n)
  sage: recurrence = lambda a,b:  111-1130/a+3000/(a*b)
  sage: gener1 = lambda n: (alpha*100^n+beta*6^n+gamma*5^n)
  sage: solGen =  lambda n: gener1(n+1)/gener1(n)

Sage example in ./sol/float.tex, line 137::

  sage: u2 = recurrence(u1,u0)
  sage: s = [u2==solGen(2),u1==solGen(1),u0==solGen(0)]
  sage: t = [s[i].substitute(u0=2,u1=-4) for i in range(0,3)]

Sage example in ./sol/float.tex, line 143::

  sage: solve(t,alpha,beta,gamma)
  [[alpha == 0, beta == -3/4*r1, gamma == r1]]

Sage example in ./sol/float.tex, line 157::

  sage: alpha=0
  sage: beta = -3/4*gamma
  sage: final=solGen(n)-recurrence(solGen(n-1),solGen(n-2))
  sage: final.simplify_full()
  0

Sage example in ./sol/float.tex, line 179::

  sage: def recur(x1,x0):
  ....:     return 111 - 1130/x1 + 3000/(x0*x1)

Sage example in ./sol/float.tex, line 190::

  sage: u0 = 2.
  sage: u1 = -4.
  sage: for i in range(1,25):
  ....:     x = recur(u1,u0)
  ....:     print((i, x))
  ....:     u0 = u1
  ....:     u1 = x
  (1, 18.5000000000000)
  (2, 9.37837837837838)
  (3, 7.80115273775217)
  (4, 7.15441448097533)
  (5, 6.80678473692481)
  (6, 6.59263276872179)
  ..................
  (23, 99.9999986592167)
  (24, 99.9999999193218)

Sage example in ./sol/float.tex, line 229::

  sage: var("x")
  x
  sage: solve(x==recurrence(x,x),x)
  [x == 100, x == 5, x == 6]

Sage example in ./sol/float.tex, line 256::

  sage: RL = RealField(5000)
  sage: u0 = RL(2)
  sage: u1 = RL(-4)
  sage: for i in range(1,2500):
  ....:     x = recur(u1,u0)
  ....:     u0 = u1
  ....:     u1= x
  sage: x
  100.00000000000000000000000000000000000000000000000000000...

Sage example in ./sol/float.tex, line 281::

  sage: u0 = 2
  sage: u1 = -4
  sage: for i in range(1,2500):
  ....:     x = recur(u1,u0)
  ....:     u0 = u1
  ....:     u1 = x
  sage: float(x)
  6.0

Sage example in ./sol/float.tex, line 325::

  sage: f = lambda x: x^2
  sage: g = lambda x: x*x
  sage: sage.rings.real_mpfi.printing_style = 'brackets'
  sage: I = RIF(-1,1)
  sage: f(I)
  [0.0000000000000000 .. 1.0000000000000000]
  sage: g(I)
  [-1.0000000000000000 .. 1.0000000000000000]
"""



# sage_setup: distribution = sagemath-repl
"""
Parsing docstrings

This module contains functions and classes that parse docstrings.

AUTHORS:

- David Roe (2012-03-27) -- initial version, based on Robert Bradshaw's code.

- Jeroen Demeyer(2014-08-28) -- much improved handling of tolerances
  using interval arithmetic (:issue:`16889`).
"""

# ****************************************************************************
#       Copyright (C) 2012-2018 David Roe <roed.math@gmail.com>
#                     2012      Robert Bradshaw <robertwb@gmail.com>
#                     2012      William Stein <wstein@gmail.com>
#                     2013      R. Andrew Ohana
#                     2013      Volker Braun
#                     2013-2018 Jeroen Demeyer <jdemeyer@cage.ugent.be>
#                     2016-2021 Frédéric Chapoton
#                     2017-2018 Erik M. Bray
#                     2020      Marc Mezzarobba
#                     2020-2023 Matthias Koeppe
#                     2022      John H. Palmieri
#                     2022      Sébastien Labbé
#                     2023      Kwankyu Lee
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

import collections.abc
import doctest
import platform
import re
from collections import defaultdict
from functools import reduce
from typing import Literal, Union, overload

from sage.misc.cachefunc import cached_function
from sage.repl.preparse import preparse, strip_string_literals
from sage.doctest.rif_tol import RIFtol, add_tolerance
from sage.doctest.marked_output import MarkedOutput
from sage.doctest.check_tolerance import (
    ToleranceExceededError, check_tolerance_real_domain,
    check_tolerance_complex_domain, float_regex)

from .external import available_software, external_software


# This is the correct pattern to match ISO/IEC 6429 ANSI escape sequences:
ansi_escape_sequence = re.compile(r"(\x1b[@-Z\\-~]|\x1b\[.*?[@-~]|\x9b.*?[@-~])")

special_optional_regex = (
    "py2|long time|not implemented|not tested|optional|needs|known bug"
)
tag_with_explanation_regex = r"((?:\w|[.])*)\s*(?:\((?P<cmd_explanation>.*?)\))?"
optional_regex = re.compile(
    rf"[^ a-z]\s*(?P<cmd>{special_optional_regex})(?:\s|[:-])*(?P<tags>(?:(?:{tag_with_explanation_regex})\s*)*)",
    re.IGNORECASE,
)
special_optional_regex = re.compile(special_optional_regex, re.IGNORECASE)
tag_with_explanation_regex = re.compile(tag_with_explanation_regex, re.IGNORECASE)

nodoctest_regex = re.compile(r'\s*(#+|%+|r"+|"+|\.\.)\s*nodoctest')
optionaltag_regex = re.compile(r"^(\w|[.])+$")
optionalfiledirective_regex = re.compile(
    r'\s*(#+|%+|r"+|"+|\.\.)\s*sage\.doctest: (.*)'
)


@overload
def parse_optional_tags(string: str) -> dict[str, Union[str, None]]:
    pass


@overload
def parse_optional_tags(
    string: str, *, return_string_sans_tags: Literal[True]
) -> tuple[dict[str, Union[str, None]], str, bool]:
    pass


def parse_optional_tags(
    string: str, *, return_string_sans_tags: bool = False
) -> Union[tuple[dict[str, Union[str, None]], str, bool], dict[str, Union[str, None]]]:
    r"""
    Return a dictionary whose keys are optional tags from the following
    set that occur in a comment on the first line of the input string.

    - ``'long time'``
    - ``'not implemented'``
    - ``'not tested'``
    - ``'known bug'`` (possible values are ``None``, ``linux`` and ``macos``)
    - ``'py2'``
    - ``'optional -- FEATURE...'`` or ``'needs FEATURE...'`` --
      the dictionary will just have the key ``'FEATURE'``

    The values, if non-``None``, are strings with optional explanations
    for a tag, which may appear in parentheses after the tag in ``string``.

    INPUT:

    - ``string`` -- string

    - ``return_string_sans_tags`` -- boolean (default: ``False``); whether to
      additionally return ``string`` with the optional tags removed but other
      comments kept and a boolean ``is_persistent``

    EXAMPLES::

        sage: from sage.doctest.parsing import parse_optional_tags
        sage: parse_optional_tags("sage: magma('2 + 2')# optional: magma")
        {'magma': None}
        sage: parse_optional_tags("sage: #optional -- mypkg")
        {'mypkg': None}
        sage: parse_optional_tags("sage: print(1)  # parentheses are optional here")
        {}
        sage: parse_optional_tags("sage: print(1)  # optional")
        {}
        sage: sorted(list(parse_optional_tags("sage: #optional -- foo bar, baz")))
        ['bar', 'foo']
        sage: parse_optional_tags("sage: #optional -- foo.bar, baz")
        {'foo.bar': None}
        sage: parse_optional_tags("sage: #needs foo.bar, baz")
        {'foo.bar': None}
        sage: sorted(list(parse_optional_tags("    sage: factor(10^(10^10) + 1) # LoNg TiME, NoT TeSTED; OptioNAL -- P4cka9e")))
        ['long time', 'not tested', 'p4cka9e']
        sage: parse_optional_tags("    sage: raise RuntimeError # known bug")
        {'bug': None}
        sage: sorted(list(parse_optional_tags("    sage: determine_meaning_of_life() # long time, not implemented")))
        ['long time', 'not implemented']

    We don't parse inside strings::

        sage: parse_optional_tags("    sage: print('  # long time')")
        {}
        sage: parse_optional_tags("    sage: print('  # long time')  # not tested")
        {'not tested': None}

    UTF-8 works::

        sage: parse_optional_tags("'ěščřžýáíéďĎ'")
        {}

    Tags with parenthesized explanations::

        sage: parse_optional_tags("    sage: 1 + 1  # long time (1 year, 2 months??), optional - bliss (because)")
        {'bliss': 'because', 'long time': '1 year, 2 months??'}

    With ``return_string_sans_tags=True``::

        sage: parse_optional_tags("sage: print(1)  # very important 1  # optional - foo",
        ....:                     return_string_sans_tags=True)
        ({'foo': None}, 'sage: print(1)  # very important 1  ', False)
        sage: parse_optional_tags("sage: print(    # very important too  # optional - foo\n....:     2)",
        ....:                     return_string_sans_tags=True)
        ({'foo': None}, 'sage: print(    # very important too  \n....:     2)', False)
        sage: parse_optional_tags("sage: #this is persistent #needs scipy",
        ....:                     return_string_sans_tags=True)
        ({'scipy': None}, 'sage: #this is persistent ', True)
        sage: parse_optional_tags("sage: #this is not #needs scipy\n....: import scipy",
        ....:                     return_string_sans_tags=True)
        ({'scipy': None}, 'sage: #this is not \n....: import scipy', False)
    """
    safe, literals, state = strip_string_literals(string)
    split = safe.split('\n', 1)
    if len(split) > 1:
        first_line, rest = split
    else:
        first_line, rest = split[0], None

    sharp_index = first_line.find('#')
    if sharp_index < 0:                  # no comment
        if return_string_sans_tags:
            return {}, string, False
        else:
            return {}

    first_line_sans_comments, comment = first_line[:sharp_index] % literals, first_line[sharp_index:] % literals
    if not first_line_sans_comments.endswith("  ") and not first_line_sans_comments.rstrip().endswith("sage:"):
        # Enforce two spaces before comment
        first_line_sans_comments = first_line_sans_comments.rstrip() + "  "

    if return_string_sans_tags:
        # skip non-tag comments that precede the first tag comment
        if m := optional_regex.search(comment):
            sharp_index = comment[:m.start(0) + 1].rfind('#')
            if sharp_index >= 0:
                first_line = first_line_sans_comments + comment[:sharp_index]
                comment = comment[sharp_index:]
        else:
            # no tag comment
            return {}, string, False

    tags: dict[str, Union[str, None]] = {}
    for m in optional_regex.finditer(comment):
        cmd = m.group("cmd").lower().strip()
        if cmd == "":
            # skip empty tags
            continue
        if cmd == "known bug":
            value = None
            if m.groups("tags") and m.group("tags").strip().lower().startswith("linux"):
                value = "linux"
            if m.groups("tags") and m.group("tags").strip().lower().startswith("macos"):
                value = "macos"

            # rename 'known bug' to 'bug' so that such tests will be run by sage -t ... -only-optional=bug
            tags["bug"] = value
        elif cmd not in ["optional", "needs"]:
            tags[cmd] = m.group("cmd_explanation") or None
        else:
            # other tags with additional values
            tags_with_value = {
                m.group(1).lower().strip(): m.group(2) or None
                for m in tag_with_explanation_regex.finditer(m.group("tags"))
            }
            tags_with_value.pop("", None)
            tags.update(tags_with_value)

    if return_string_sans_tags:
        is_persistent = tags and first_line_sans_comments.strip() == 'sage:' and not rest  # persistent (block-scoped) tag
        return tags, (first_line + '\n' + rest % literals if rest is not None
                      else first_line), is_persistent
    else:
        return tags


def parse_file_optional_tags(lines):
    r"""
    Scan the first few lines for file-level doctest directives.

    INPUT:

    - ``lines`` -- iterable of pairs ``(lineno, line)``

    OUTPUT: dictionary whose keys are strings (tags);
    see :func:`parse_optional_tags`

    EXAMPLES::

        sage: from sage.doctest.parsing import parse_file_optional_tags
        sage: filename = tmp_filename(ext='.pyx')
        sage: with open(filename, "r") as f:
        ....:     parse_file_optional_tags(enumerate(f))
        {}
        sage: with open(filename, "w") as f:
        ....:     _ = f.write("# nodoctest")
        sage: with open(filename, "r") as f:
        ....:     parse_file_optional_tags(enumerate(f))
        {'not tested': None}
        sage: with open(filename, "w") as f:
        ....:     _ = f.write("# sage.doctest: "    # broken in two source lines to avoid the pattern
        ....:                 "optional - xyz")     # of relint (multiline_doctest_comment)
        sage: with open(filename, "r") as f:
        ....:     parse_file_optional_tags(enumerate(f))
        {'xyz': None}
    """
    tags = {}
    for line_count, line in lines:
        if nodoctest_regex.match(line):
            tags['not tested'] = None
        if m := optionalfiledirective_regex.match(line):
            file_tag_string = m.group(2)
            tags.update(parse_optional_tags('#' + file_tag_string))
        if line_count >= 10:
            break
    return tags


@cached_function
def _standard_tags():
    r"""
    Return the set of the names of all standard features.

    EXAMPLES::

        sage: from sage.doctest.parsing import _standard_tags
        sage: sorted(_standard_tags())
        [..., 'numpy', ..., 'sage.rings.finite_rings', ...]
    """
    from sage.features.all import all_features
    return frozenset(feature.name for feature in all_features()
                     if feature._spkg_type() == 'standard')


def _tag_group(tag):
    r"""
    Classify a doctest tag as belonging to one of 4 groups.

    INPUT:

    - ``tag`` -- string

    OUTPUT: string; one of ``'special'``, ``'optional'``, ``'standard'``, ``'sage'``

    EXAMPLES::

        sage: from sage.doctest.parsing import _tag_group
        sage: _tag_group('scipy')
        'standard'
        sage: _tag_group('sage.numerical.mip')
        'sage'
        sage: _tag_group('bliss')
        'optional'
        sage: _tag_group('not tested')
        'special'
    """
    if tag.startswith('sage.'):
        return 'sage'
    elif tag in _standard_tags():
        return 'standard'
    elif not special_optional_regex.fullmatch(tag):
        return 'optional'
    else:
        return 'special'


def unparse_optional_tags(tags, prefix='# '):
    r"""
    Return a comment string that sets ``tags``.

    INPUT:

    - ``tags`` -- dictionary or iterable of tags, as output by
      :func:`parse_optional_tags`

    - ``prefix`` -- to be put before a nonempty string

    EXAMPLES::

        sage: from sage.doctest.parsing import unparse_optional_tags
        sage: unparse_optional_tags({})
        ''
        sage: unparse_optional_tags({'magma': None})
        '# optional - magma'
        sage: unparse_optional_tags({'fictional_optional': None,
        ....:                        'sage.rings.number_field': None,
        ....:                        'scipy': 'just because',
        ....:                        'bliss': None})
        '# optional - bliss fictional_optional, needs scipy (just because) sage.rings.number_field'
        sage: unparse_optional_tags(['long time', 'not tested', 'p4cka9e'], prefix='')
        'long time, not tested, optional - p4cka9e'
    """
    group = defaultdict(set)
    if isinstance(tags, collections.abc.Mapping):
        for tag, explanation in tags.items():
            if tag == 'bug':
                tag = 'known bug'
            group[_tag_group(tag)].add(f'{tag} ({explanation})' if explanation else tag)
    else:
        for tag in tags:
            if tag == 'bug':
                tag = 'known bug'
            group[_tag_group(tag)].add(tag)

    tags = sorted(group.pop('special', []))
    if 'optional' in group:
        tags.append('optional - ' + " ".join(sorted(group.pop('optional'))))
    if 'standard' in group or 'sage' in group:
        tags.append('needs ' + " ".join(sorted(group.pop('standard', []))
                                        + sorted(group.pop('sage', []))))
    assert not group
    if tags:
        return prefix + ', '.join(tags)
    return ''


optional_tag_columns = [48, 56, 64, 72, 80, 84]
standard_tag_columns = [88, 100, 120, 160]


def update_optional_tags(line, tags=None, *, add_tags=None, remove_tags=None, force_rewrite=False):
    r"""
    Return the doctest ``line`` with tags changed.

    EXAMPLES::

        sage: from sage.doctest.parsing import update_optional_tags, optional_tag_columns, standard_tag_columns
        sage: ruler = ''
        sage: for column in optional_tag_columns:
        ....:     ruler += ' ' * (column - len(ruler)) + 'V'
        sage: for column in standard_tag_columns:
        ....:     ruler += ' ' * (column - len(ruler)) + 'v'
        sage: def print_with_ruler(lines):
        ....:     print('|' + ruler)
        ....:     for line in lines:
        ....:         print('|' + line)
        sage: print_with_ruler([  # the tags are obscured in the source file to avoid relint warnings
        ....:     update_optional_tags('    sage: something()  # opt' 'ional - latte_int',
        ....:                          remove_tags=['latte_int', 'wasnt_even_there']),
        ....:     update_optional_tags('    sage: nothing_to_be_seen_here()',
        ....:                          tags=['scipy', 'long time']),
        ....:     update_optional_tags('    sage: nothing_to_be_seen_here(honestly=True)',
        ....:                          add_tags=['scipy', 'long time']),
        ....:     update_optional_tags('    sage: nothing_to_be_seen_here(honestly=True, very=True)',
        ....:                          add_tags=['scipy', 'long time']),
        ....:     update_optional_tags('    sage: no_there_is_absolutely_nothing_to_be_seen_here_i_am_serious()#opt' 'ional:bliss',
        ....:                          add_tags=['scipy', 'long time']),
        ....:     update_optional_tags('    sage: ntbsh()  # abbrv for above#opt' 'ional:bliss',
        ....:                          add_tags={'scipy': None, 'long time': '30s on the highest setting'}),
        ....:     update_optional_tags('    sage: no_there_is_absolutely_nothing_to_be_seen_here_i_am_serious()  # really, you can trust me here',
        ....:                          add_tags=['scipy']),
        ....: ])
        |                                                V       V       V       V       V   V   v           v                   v                                       v
        |    sage: something()
        |    sage: nothing_to_be_seen_here()             # long time                             # needs scipy
        |    sage: nothing_to_be_seen_here(honestly=True)        # long time                     # needs scipy
        |    sage: nothing_to_be_seen_here(honestly=True, very=True)     # long time             # needs scipy
        |    sage: no_there_is_absolutely_nothing_to_be_seen_here_i_am_serious()         # long time, optional - bliss, needs scipy
        |    sage: ntbsh()  # abbrv for above            # long time (30s on the highest setting), optional - bliss, needs scipy
        |    sage: no_there_is_absolutely_nothing_to_be_seen_here_i_am_serious()  # really, you can trust me here                # needs scipy

    When no tags are changed, by default, the unchanged input is returned.
    We can force a rewrite; unconditionally or whenever standard tags are involved.
    But even when forced, if comments are already aligned at one of the standard alignment columns,
    this alignment is kept even if we would normally realign farther to the left::

        sage: print_with_ruler([
        ....:     update_optional_tags('    sage: unforced()       # opt' 'ional - latte_int'),
        ....:     update_optional_tags('    sage: unforced()  # opt' 'ional - latte_int',
        ....:                          add_tags=['latte_int']),
        ....:     update_optional_tags('    sage: forced()#opt' 'ional- latte_int',
        ....:                          force_rewrite=True),
        ....:     update_optional_tags('    sage: forced()  # opt' 'ional - scipy',
        ....:                          force_rewrite='standard'),
        ....:     update_optional_tags('    sage: aligned_with_below()                                  # opt' 'ional - 4ti2',
        ....:                          force_rewrite=True),
        ....:     update_optional_tags('    sage: aligned_with_above()                                  # opt' 'ional - 4ti2',
        ....:                          force_rewrite=True),
        ....:     update_optional_tags('    sage: also_already_aligned()                                                                                        # ne' 'eds scipy',
        ....:                          force_rewrite='standard'),
        ....:     update_optional_tags('    sage: two_columns_first_preserved()         # lo' 'ng time                             # ne' 'eds scipy',
        ....:                          force_rewrite='standard'),
        ....:     update_optional_tags('    sage: two_columns_first_preserved()                 # lo' 'ng time                                 # ne' 'eds scipy',
        ....:                          force_rewrite='standard'),
        ....: ])
        |                                                V       V       V       V       V   V   v           v                   v                                       v
        |    sage: unforced()       # optional - latte_int
        |    sage: unforced()  # optional - latte_int
        |    sage: forced()                              # optional - latte_int
        |    sage: forced()                                                                      # needs scipy
        |    sage: aligned_with_below()                                  # optional - 4ti2
        |    sage: aligned_with_above()                                  # optional - 4ti2
        |    sage: also_already_aligned()                                                                                        # needs scipy
        |    sage: two_columns_first_preserved()         # long time                             # needs scipy
        |    sage: two_columns_first_preserved()                 # long time                     # needs scipy

    Rewriting a persistent (block-scoped) tag::

        sage: print_with_ruler([
        ....:     update_optional_tags('    sage:    #opt' 'ional:magma sage.symbolic',
        ....:                          force_rewrite=True),
        ....: ])
        |                                                V       V       V       V       V   V   v           v                   v                                       v
        |    sage: # optional - magma, needs sage.symbolic
    """
    if not re.match('( *sage: *)(.*)', line):
        raise ValueError(f'line must start with a sage: prompt, got: {line}')

    current_tags, line_sans_tags, is_persistent = parse_optional_tags(line.rstrip(), return_string_sans_tags=True)

    if isinstance(tags, collections.abc.Mapping):
        new_tags = dict(tags)
    elif tags is not None:
        new_tags = {tag: None for tag in tags}
    else:
        new_tags = dict(current_tags)

    if add_tags is not None:
        if isinstance(add_tags, collections.abc.Mapping):
            new_tags.update(add_tags)
        else:
            new_tags.update({tag: None for tag in add_tags})

    if remove_tags is not None:
        for tag in remove_tags:
            new_tags.pop(tag, None)

    if not force_rewrite and new_tags == current_tags:
        return line

    if not new_tags:
        return line_sans_tags.rstrip()

    if (force_rewrite == 'standard'
            and new_tags == current_tags
            and not any(_tag_group(tag) in ['standard', 'sage']
                        for tag in new_tags)):
        return line

    if is_persistent:
        line = line_sans_tags.rstrip() + ' '
    else:
        group = defaultdict(set)
        for tag in new_tags:
            group[_tag_group(tag)].add(tag)
        tag_columns = (optional_tag_columns if group['optional'] or group['special']
                       else standard_tag_columns)

        if len(line_sans_tags) in tag_columns and line_sans_tags[-2:] == '  ':
            # keep alignment
            line = line_sans_tags
            pass
        else:
            # realign
            line = line_sans_tags.rstrip()
            for column in tag_columns:
                if len(line) <= column - 2:
                    line += ' ' * (column - 2 - len(line))
                    break
            line += '  '

        if (group['optional'] or group['special']) and (group['standard'] or group['sage']):
            # Try if two-column mode works better
            first_part = unparse_optional_tags({tag: explanation
                                                for tag, explanation in new_tags.items()
                                                if (tag in group['optional']
                                                    or tag in group['special'])})
            column = standard_tag_columns[0]
            if len(line + first_part) + 8 <= column:
                line += first_part
                line += ' ' * (column - len(line))
                line += unparse_optional_tags({tag: explanation
                                               for tag, explanation in new_tags.items()
                                               if not (tag in group['optional']
                                                       or tag in group['special'])})
                return line.rstrip()

    line += unparse_optional_tags(new_tags)
    return line


def parse_tolerance(source, want):
    r"""
    Return a version of ``want`` marked up with the tolerance tags
    specified in ``source``.

    INPUT:

    - ``source`` -- string, the source of a doctest
    - ``want`` -- string, the desired output of the doctest

    OUTPUT: ``want`` if there are no tolerance tags specified; a
    :class:`MarkedOutput` version otherwise

    EXAMPLES::

        sage: from sage.doctest.parsing import parse_tolerance
        sage: marked = parse_tolerance("sage: s.update(abs_tol = .0000001)", "")
        sage: type(marked)
        <class 'str'>
        sage: marked = parse_tolerance("sage: s.update(tol = 0.1); s.rel_tol # abs tol     0.01 ", "")
        sage: marked.tol
        0
        sage: marked.rel_tol
        0
        sage: marked.abs_tol                                                            # needs sage.rings.real_mpfr
        0.010000000000000000000...?
    """
    # regular expressions
    random_marker = re.compile('.*random', re.I)
    tolerance_pattern = re.compile(r'\b((?:abs(?:olute)?)|(?:rel(?:ative)?))? *?tol(?:erance)?\b( +[0-9.e+-]+)?')

    safe, literals, state = strip_string_literals(source)
    first_line = safe.split('\n', 1)[0]
    if '#' not in first_line:
        return want
    comment = first_line[first_line.find('#') + 1:]
    comment = comment[comment.index('(') + 1: comment.rindex(')')]
    # strip_string_literals replaces comments
    comment = literals[comment]
    if random_marker.search(comment):
        want = MarkedOutput(want).update(random=True)
    else:
        m = tolerance_pattern.search(comment)
        if m:
            rel_or_abs, epsilon = m.groups()
            if epsilon is None:
                epsilon = RIFtol("1e-15")
            else:
                epsilon = RIFtol(epsilon)
            if rel_or_abs is None:
                want = MarkedOutput(want).update(tol=epsilon)
            elif rel_or_abs.startswith('rel'):
                want = MarkedOutput(want).update(rel_tol=epsilon)
            elif rel_or_abs.startswith('abs'):
                want = MarkedOutput(want).update(abs_tol=epsilon)
            else:
                raise RuntimeError
    return want


def pre_hash(s):
    """
    Prepends a string with its length.

    EXAMPLES::

        sage: from sage.doctest.parsing import pre_hash
        sage: pre_hash("abc")
        '3:abc'
    """
    return "%s:%s" % (len(s), s)


def get_source(example):
    """
    Return the source with the leading 'sage: ' stripped off.

    EXAMPLES::

        sage: from sage.doctest.parsing import get_source
        sage: from sage.doctest.sources import DictAsObject
        sage: example = DictAsObject({})
        sage: example.sage_source = "2 + 2"
        sage: example.source = "sage: 2 + 2"
        sage: get_source(example)
        '2 + 2'
        sage: example = DictAsObject({})
        sage: example.source = "3 + 3"
        sage: get_source(example)
        '3 + 3'
    """
    return getattr(example, 'sage_source', example.source)


def reduce_hex(fingerprints):
    """
    Return a symmetric function of the arguments as hex strings.

    The arguments should be 32 character strings consisting of hex
    digits: 0-9 and a-f.

    EXAMPLES::

        sage: from sage.doctest.parsing import reduce_hex
        sage: reduce_hex(["abc", "12399aedf"])
        '0000000000000000000000012399a463'
        sage: reduce_hex(["12399aedf","abc"])
        '0000000000000000000000012399a463'
    """
    from operator import xor
    res = reduce(xor, (int(x, 16) for x in fingerprints), 0)
    if res < 0:
        res += 1 << 128
    return "%032x" % res


class OriginalSource():
    r"""
    Context swapping out the pre-parsed source with the original for
    better reporting.

    EXAMPLES::

        sage: from sage.doctest.sources import FileDocTestSource
        sage: from sage.doctest.control import DocTestDefaults
        sage: filename = sage.doctest.forker.__file__
        sage: FDS = FileDocTestSource(filename, DocTestDefaults())
        sage: doctests, extras = FDS.create_doctests(globals())
        sage: ex = doctests[0].examples[0]
        sage: ex.sage_source
        'doctest_var = 42; doctest_var^2\n'
        sage: ex.source
        'doctest_var = Integer(42); doctest_var**Integer(2)\n'
        sage: from sage.doctest.parsing import OriginalSource
        sage: with OriginalSource(ex):
        ....:     ex.source
        'doctest_var = 42; doctest_var^2\n'
    """
    def __init__(self, example):
        """
        Swaps out the source for the sage_source of a doctest example.

        INPUT:

        - ``example`` -- a :class:`doctest.Example` instance

        EXAMPLES::

            sage: from sage.doctest.sources import FileDocTestSource
            sage: from sage.doctest.control import DocTestDefaults
            sage: filename = sage.doctest.forker.__file__
            sage: FDS = FileDocTestSource(filename, DocTestDefaults())
            sage: doctests, extras = FDS.create_doctests(globals())
            sage: ex = doctests[0].examples[0]
            sage: from sage.doctest.parsing import OriginalSource
            sage: OriginalSource(ex)
            <sage.doctest.parsing.OriginalSource object at ...>
        """
        self.example = example

    def __enter__(self):
        r"""
        EXAMPLES::

            sage: from sage.doctest.sources import FileDocTestSource
            sage: from sage.doctest.control import DocTestDefaults
            sage: filename = sage.doctest.forker.__file__
            sage: FDS = FileDocTestSource(filename, DocTestDefaults())
            sage: doctests, extras = FDS.create_doctests(globals())
            sage: ex = doctests[0].examples[0]
            sage: from sage.doctest.parsing import OriginalSource
            sage: with OriginalSource(ex): # indirect doctest
            ....:     ex.source
            'doctest_var = 42; doctest_var^2\n'
        """
        if hasattr(self.example, 'sage_source'):
            self.old_source, self.example.source = self.example.source, self.example.sage_source

    def __exit__(self, *args):
        r"""
        EXAMPLES::

            sage: from sage.doctest.sources import FileDocTestSource
            sage: from sage.doctest.control import DocTestDefaults
            sage: filename = sage.doctest.forker.__file__
            sage: FDS = FileDocTestSource(filename, DocTestDefaults())
            sage: doctests, extras = FDS.create_doctests(globals())
            sage: ex = doctests[0].examples[0]
            sage: from sage.doctest.parsing import OriginalSource
            sage: with OriginalSource(ex): # indirect doctest
            ....:     ex.source
            'doctest_var = 42; doctest_var^2\n'
            sage: ex.source # indirect doctest
            'doctest_var = Integer(42); doctest_var**Integer(2)\n'
        """
        if hasattr(self.example, 'sage_source'):
            self.example.source = self.old_source


class SageDocTestParser(doctest.DocTestParser):
    """
    A version of the standard doctest parser which handles Sage's
    custom options and tolerances in floating point arithmetic.
    """

    long: bool
    file_optional_tags: set[str]
    optional_tags: Union[bool, set[str]]
    optional_only: bool
    optionals: dict[str, int]
    probed_tags: Union[bool, set[str]]

    def __init__(self, optional_tags=(), long=False, *, probed_tags=(), file_optional_tags=()):
        r"""
        INPUT:

        - ``optional_tags`` -- list or tuple of strings
        - ``long`` -- boolean, whether to run doctests marked as taking a
          long time
        - ``probed_tags`` -- list or tuple of strings
        - ``file_optional_tags`` -- an iterable of strings

        EXAMPLES::

            sage: from sage.doctest.parsing import SageDocTestParser
            sage: DTP = SageDocTestParser(('sage','magma','guava'))
            sage: ex = DTP.parse("sage: 2 + 2\n")[1]
            sage: ex.sage_source
            '2 + 2\n'
            sage: ex = DTP.parse("sage: R.<x> = ZZ[]")[1]
            sage: ex.source
            "R = ZZ['x']; (x,) = R._first_ngens(1)\n"

        TESTS::

            sage: TestSuite(DTP).run()
        """
        self.long = long
        self.optionals = defaultdict(int)  # record skipped optional tests
        if optional_tags is True:  # run all optional tests
            self.optional_tags = True
            self.optional_only = False
        else:
            self.optional_tags = set(optional_tags)
            if 'sage' in self.optional_tags:
                self.optional_only = False
                self.optional_tags.remove('sage')
            else:
                self.optional_only = True
        if probed_tags is True:
            self.probed_tags = True
        else:
            self.probed_tags = set(probed_tags)
        self.file_optional_tags = set(file_optional_tags)

    def __eq__(self, other):
        """
        Comparison.

        EXAMPLES::

            sage: from sage.doctest.parsing import SageDocTestParser
            sage: DTP = SageDocTestParser(('sage','magma','guava'), True)
            sage: DTP2 = SageDocTestParser(('sage','magma','guava'), False)
            sage: DTP == DTP2
            False
        """
        if not isinstance(other, SageDocTestParser):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Test for non-equality.

        EXAMPLES::

            sage: from sage.doctest.parsing import SageDocTestParser
            sage: DTP = SageDocTestParser(('sage','magma','guava'), True)
            sage: DTP2 = SageDocTestParser(('sage','magma','guava'), False)
            sage: DTP != DTP2
            True
        """
        return not (self == other)

    def parse(self, string, *args):
        r"""
        A Sage specialization of :class:`doctest.DocTestParser`.

        INPUT:

        - ``string`` -- the string to parse
        - ``name`` -- (optional) string giving the name identifying string,
          to be used in error messages

        OUTPUT:

        - A list consisting of strings and :class:`doctest.Example`
          instances.  There will be at least one string between
          successive examples (exactly one unless long or optional
          tests are removed), and it will begin and end with a string.

        EXAMPLES::

            sage: from sage.doctest.parsing import SageDocTestParser
            sage: DTP = SageDocTestParser(('sage','magma','guava'))
            sage: example = 'Explanatory text::\n\n    sage: E = magma("EllipticCurve([1, 1, 1, -10, -10])") # optional: magma\n\nLater text'
            sage: parsed = DTP.parse(example)
            sage: parsed[0]
            'Explanatory text::\n\n'
            sage: parsed[1].sage_source
            'E = magma("EllipticCurve([1, 1, 1, -10, -10])") # optional: magma\n'
            sage: parsed[2]
            '\nLater text'

        If the doctest parser is not created to accept a given
        optional argument, the corresponding examples will just be
        removed::

            sage: DTP2 = SageDocTestParser(('sage',))
            sage: parsed2 = DTP2.parse(example)
            sage: parsed2
            ['Explanatory text::\n\n', '\nLater text']

        You can mark doctests as having a particular tolerance::

            sage: example2 = 'sage: gamma(1.6) # tol 2.0e-11\n0.893515349287690'
            sage: ex = DTP.parse(example2)[1]
            sage: ex.sage_source
            'gamma(1.6) # tol 2.0e-11\n'
            sage: ex.want
            '0.893515349287690\n'
            sage: type(ex.want)
            <class 'sage.doctest.marked_output.MarkedOutput'>
            sage: ex.want.tol                                                           # needs sage.rings.real_interval_field
            2.000000000000000000...?e-11

        You can use continuation lines::

            sage: s = "sage: for i in range(4):\n....:     print(i)\n....:\n"
            sage: ex = DTP2.parse(s)[1]
            sage: ex.source
            'for i in range(Integer(4)):\n    print(i)\n'

        Sage currently accepts backslashes as indicating that the end
        of the current line should be joined to the next line.  This
        feature allows for breaking large integers over multiple lines
        but is not standard for Python doctesting.  It's not
        guaranteed to persist::

            sage: n = 1234\
            ....:     5678
            sage: print(n)
            12345678
            sage: type(n)
            <class 'sage.rings.integer.Integer'>

        It also works without the line continuation::

            sage: m = 8765\
            4321
            sage: print(m)
            87654321

        Optional tags at the start of an example block persist to the end of
        the block (delimited by a blank line)::

            sage: # long time, needs sage.rings.number_field
            sage: QQbar(I)^10000
            1
            sage: QQbar(I)^10000  # not tested
            I

            sage: # needs sage.rings.finite_rings
            sage: GF(7)
            Finite Field of size 7
            sage: GF(10)
            Traceback (most recent call last):
            ...
            ValueError: the order of a finite field must be a prime power

        Test that :issue:`26575` is resolved::

            sage: example3 = 'sage: Zp(5,4,print_mode="digits")(5)\n...00010'
            sage: parsed3 = DTP.parse(example3)
            sage: dte = parsed3[1]
            sage: dte.sage_source
            'Zp(5,4,print_mode="digits")(5)\n'
            sage: dte.want
            '...00010\n'

        Style warnings::

            sage: def parse(test_string):
            ....:     return [x if isinstance(x, str)
            ....:               else (getattr(x, 'warnings', None), x.sage_source, x.source)
            ....:             for x in DTP.parse(test_string)]

            sage: parse('sage: 1 # optional guava mango\nsage: 2 # optional guava\nsage: 3 # optional guava\nsage: 4 # optional guava\nsage: 5 # optional guava\n\nsage: 11 # optional guava')
            ['',
             (["Consider using a block-scoped tag by inserting the line 'sage: # optional - guava' just before this line to avoid repeating the tag 5 times"],
              '1 # optional guava mango\n',
              'None  # virtual doctest'),
             '',
             (None, '2 # optional guava\n', 'Integer(2) # optional guava\n'),
             '',
             (None, '3 # optional guava\n', 'Integer(3) # optional guava\n'),
             '',
             (None, '4 # optional guava\n', 'Integer(4) # optional guava\n'),
             '',
             (None, '5 # optional guava\n', 'Integer(5) # optional guava\n'),
             '\n',
             (None, '11 # optional guava\n', 'Integer(11) # optional guava\n'),
             '']

            sage: parse('sage: 1 # optional guava\nsage: 2 # optional guava mango\nsage: 3 # optional guava\nsage: 4 # optional guava\nsage: 5 # optional guava\n')
            ['',
             (["Consider using a block-scoped tag by inserting the line 'sage: # optional - guava' just before this line to avoid repeating the tag 5 times"],
              '1 # optional guava\n',
              'Integer(1) # optional guava\n'),
             '',
             '',
             (None, '3 # optional guava\n', 'Integer(3) # optional guava\n'),
             '',
             (None, '4 # optional guava\n', 'Integer(4) # optional guava\n'),
             '',
             (None, '5 # optional guava\n', 'Integer(5) # optional guava\n'),
             '']

            sage: parse('sage: # optional mango\nsage: 1 # optional guava\nsage: 2 # optional guava mango\nsage: 3 # optional guava\nsage: 4 # optional guava\n sage: 5 # optional guava\n')  # optional - guava mango
            ['',
             (["Consider updating this block-scoped tag to 'sage: # optional - guava mango' to avoid repeating the tag 5 times"],
              '# optional mango\n',
              'None  # virtual doctest'),
             '',
             '',
             '',
             '',
             '',
             '']

            sage: parse('::\n\n    sage: 1 # optional guava\n    sage: 2 # optional guava mango\n    sage: 3 # optional guava\n\n::\n\n    sage: 4 # optional guava\n     sage: 5 # optional guava\n')
            ['::\n\n',
            (None, '1 # optional guava\n', 'Integer(1) # optional guava\n'),
            '',
            '',
            (None, '3 # optional guava\n', 'Integer(3) # optional guava\n'),
            '\n::\n\n',
            (None, '4 # optional guava\n', 'Integer(4) # optional guava\n'),
            '',
            (None, '5 # optional guava\n', 'Integer(5) # optional guava\n'),
            '']

        TESTS::

            sage: parse("::\n\n    sage: # needs sage.combinat\n    sage: from sage.geometry.polyhedron.combinatorial_polyhedron.conversions \\\n    ....:         import incidence_matrix_to_bit_rep_of_Vrep\n    sage: P = polytopes.associahedron(['A',3])\n\n")
            ['::\n\n',
            '',
            (None,
            'from sage.geometry.polyhedron.combinatorial_polyhedron.conversions import incidence_matrix_to_bit_rep_of_Vrep\n',
            'from sage.geometry.polyhedron.combinatorial_polyhedron.conversions import incidence_matrix_to_bit_rep_of_Vrep\n'),
            '',
            (None,
            "P = polytopes.associahedron(['A',3])\n",
            "P = polytopes.associahedron(['A',Integer(3)])\n"),
            '\n']

            sage: example4 = '::\n\n        sage: C.minimum_distance(algorithm="guava")  # optional - guava\n        ...\n        24\n\n'
            sage: parsed4 = DTP.parse(example4)
            sage: dte = parsed4[1]
            sage: dte.sage_source
            'C.minimum_distance(algorithm="guava")  # optional - guava\n'
            sage: dte.want
            '...\n24\n'
        """
        # Regular expressions
        find_sage_prompt = re.compile(r"^(\s*)sage: ", re.M)
        find_sage_continuation = re.compile(r"^(\s*)\.\.\.\.:", re.M)
        find_python_continuation = re.compile(r"^(\s*)\.\.\.([^\.])", re.M)
        python_prompt = re.compile(r"^(\s*)>>>", re.M)
        backslash_replacer = re.compile(r"""(\s*)sage:(.*)\\\ *
\ *((\.){4}:)?\ *""")

        # The following are used to allow ... at the beginning of output
        ellipsis_tag = "<TEMP_ELLIPSIS_TAG>"

        # Hack for non-standard backslash line escapes accepted by the current
        # doctest system.
        m = backslash_replacer.search(string)
        while m is not None:
            g = m.groups()
            string = string[:m.start()] + g[0] + "sage:" + g[1] + string[m.end():]
            m = backslash_replacer.search(string, m.start())

        replace_ellipsis = not python_prompt.search(string)
        if replace_ellipsis:
            # There are no >>> prompts, so we can allow ... to begin the output
            # We do so by replacing ellipses with a special tag, then putting them back after parsing
            string = find_python_continuation.sub(r"\1" + ellipsis_tag + r"\2", string)
        string = find_sage_prompt.sub(r"\1>>> sage: ", string)
        string = find_sage_continuation.sub(r"\1...", string)
        res = doctest.DocTestParser.parse(self, string, *args)
        filtered = []
        persistent_optional_tags = self.file_optional_tags
        persistent_optional_tag_setter = None
        persistent_optional_tag_setter_index = None
        first_example_in_block = None
        first_example_in_block_index = None
        tag_count_within_block = defaultdict(lambda: 0)

        def update_tag_counts(optional_tags):
            for tag in optional_tags:
                if tag not in persistent_optional_tags:
                    tag_count_within_block[tag] += 1
            tag_count_within_block[''] += 1

        def check_and_clear_tag_counts():
            if (num_examples := tag_count_within_block['']) >= 4:
                if overused_tags := {tag for tag, count in tag_count_within_block.items()
                                     if tag and count >= num_examples}:
                    overused_tags.update(persistent_optional_tags)
                    overused_tags.difference_update(self.file_optional_tags)
                    suggested = unparse_optional_tags(overused_tags, prefix='sage: # ')

                    if persistent_optional_tag_setter:
                        warning_example = persistent_optional_tag_setter
                        index = persistent_optional_tag_setter_index
                        warning = (f"Consider updating this block-scoped tag to '{suggested}' "
                                   f"to avoid repeating the tag {num_examples} times")
                    else:
                        warning_example = first_example_in_block
                        index = first_example_in_block_index
                        warning = (f"Consider using a block-scoped tag by "
                                   f"inserting the line '{suggested}' just before this line "
                                   f"to avoid repeating the tag {num_examples} times")

                    if not (index < len(filtered) and filtered[index] == warning_example):
                        # The example to which we want to attach our warning is
                        # not in ``filtered``. It is either the persistent tag line,
                        # or the first example of the block and not run because of unmet tags,
                        # or just a comment. Either way, we transform this example
                        # to a virtual example and attach the warning to it.
                        warning_example.sage_source = warning_example.source
                        if warning_example.sage_source.startswith("sage: "):
                            warning_example.sage_source = warning_example.source[6:]
                        warning_example.source = 'None  # virtual doctest'
                        warning_example.want = ''
                        filtered.insert(index, warning_example)

                    if not hasattr(warning_example, 'warnings'):
                        warning_example.warnings = []
                    warning_example.warnings.append(warning)
            tag_count_within_block.clear()

        for item in res:
            if isinstance(item, doctest.Example):
                optional_tags_with_values, _, is_persistent = parse_optional_tags(item.source, return_string_sans_tags=True)
                optional_tags = set(optional_tags_with_values)
                if is_persistent:
                    check_and_clear_tag_counts()
                    persistent_optional_tags = optional_tags
                    persistent_optional_tags.update(self.file_optional_tags)
                    persistent_optional_tag_setter = first_example_in_block = item
                    persistent_optional_tag_setter_index = len(filtered)
                    first_example_in_block_index = None
                    continue

                if not first_example_in_block:
                    first_example_in_block = item
                    first_example_in_block_index = len(filtered)
                update_tag_counts(optional_tags)
                optional_tags.update(persistent_optional_tags)
                item.optional_tags = frozenset(optional_tags)
                item.probed_tags = set()
                if optional_tags:
                    for tag in optional_tags:
                        self.optionals[tag] += 1
                    if (('not implemented' in optional_tags) or
                            ('not tested' in optional_tags)):
                        continue

                    if 'long time' in optional_tags:
                        if self.long:
                            optional_tags.remove('long time')
                        else:
                            continue

                    if self.optional_tags is not True:
                        extra = {
                            tag
                            for tag in optional_tags
                            if (
                                tag not in self.optional_tags
                                and tag not in available_software
                            )
                        }
                        if extra and any(tag in ["bug"] for tag in extra):
                            # Bug only occurs on a specific platform?
                            bug_platform = optional_tags_with_values.get("bug")
                            # System platform as either linux or macos
                            system_platform = (
                                platform.system().lower().replace("darwin", "macos")
                            )
                            if not bug_platform or bug_platform == system_platform:
                                continue
                        elif extra:
                            if any(tag in external_software for tag in extra):
                                # never probe "external" software
                                continue
                            if any(tag in ['webbrowser'] for tag in extra):
                                # never probe
                                continue
                            if any(tag in ['got', 'expected', 'nameerror'] for tag in extra):
                                # never probe special tags added by sage-fixdoctests
                                continue
                            if all(tag in persistent_optional_tags for tag in extra):
                                # don't probe if test is only conditional
                                # on file-level or block-level tags
                                continue
                            if self.probed_tags is True:
                                item.probed_tags = extra
                            elif all(tag in self.probed_tags for tag in extra):
                                item.probed_tags = extra
                            else:
                                continue
                elif self.optional_only:
                    self.optionals['sage'] += 1
                    continue

                if replace_ellipsis:
                    item.want = item.want.replace(ellipsis_tag, "...")
                    if item.exc_msg is not None:
                        item.exc_msg = item.exc_msg.replace(ellipsis_tag, "...")
                item.want = parse_tolerance(item.source, item.want)
                if item.source.startswith("sage: "):
                    item.sage_source = item.source[6:]
                    if item.sage_source.lstrip().startswith('#'):
                        continue
                    item.source = preparse(item.sage_source)
            else:
                if '\n' in item:
                    check_and_clear_tag_counts()
                    persistent_optional_tags = self.file_optional_tags
                    persistent_optional_tag_setter = first_example_in_block = None
                    persistent_optional_tag_setter_index = first_example_in_block_index = None
            filtered.append(item)

        check_and_clear_tag_counts()

        return filtered


class SageOutputChecker(doctest.OutputChecker):
    r"""
    A modification of the doctest OutputChecker that can check
    relative and absolute tolerance of answers.

    EXAMPLES::

        sage: from sage.doctest.parsing import SageOutputChecker, MarkedOutput, SageDocTestParser
        sage: import doctest
        sage: optflag = doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS
        sage: DTP = SageDocTestParser(('sage','magma','guava'))
        sage: OC = SageOutputChecker()
        sage: example2 = 'sage: gamma(1.6) # tol 2.0e-11\n0.893515349287690'
        sage: ex = DTP.parse(example2)[1]
        sage: ex.sage_source
        'gamma(1.6) # tol 2.0e-11\n'
        sage: ex.want
        '0.893515349287690\n'
        sage: type(ex.want)
        <class 'sage.doctest.marked_output.MarkedOutput'>

        sage: # needs sage.rings.real_interval_field
        sage: ex.want.tol
        2.000000000000000000...?e-11
        sage: OC.check_output(ex.want, '0.893515349287690', optflag)
        True
        sage: OC.check_output(ex.want, '0.8935153492877', optflag)
        True
        sage: OC.check_output(ex.want, '0', optflag)
        False
        sage: OC.check_output(ex.want, 'x + 0.8935153492877', optflag)
        False
    """
    def human_readable_escape_sequences(self, string):
        r"""
        Make ANSI escape sequences human readable.

        EXAMPLES::

            sage: print('This is \x1b[1mbold\x1b[0m text')
            This is <CSI-1m>bold<CSI-0m> text

        TESTS::

            sage: from sage.doctest.parsing import SageOutputChecker
            sage: OC = SageOutputChecker()
            sage: teststr = '-'.join([
            ....:     'bold\x1b[1m',
            ....:     'red\x1b[31m',
            ....:     'oscmd\x1ba'])
            sage: OC.human_readable_escape_sequences(teststr)
            'bold<CSI-1m>-red<CSI-31m>-oscmd<ESC-a>'
        """
        def human_readable(match):
            ansi_escape = match.group(1)
            assert len(ansi_escape) >= 2
            if len(ansi_escape) == 2:
                return '<ESC-' + ansi_escape[1] + '>'
            return '<CSI-' + ansi_escape.lstrip('\x1b[\x9b') + '>'
        return ansi_escape_sequence.subn(human_readable, string)[0]

    def check_output(self, want, got, optionflags):
        r"""
        Check to see if the output matches the desired output.

        If ``want`` is a :class:`MarkedOutput` instance, takes into account the desired tolerance.

        INPUT:

        - ``want`` -- string or :class:`MarkedOutput`
        - ``got`` -- string
        - ``optionflags`` -- integer; passed down to :class:`doctest.OutputChecker`

        OUTPUT: boolean; whether ``got`` matches ``want`` up to the specified
        tolerance

        EXAMPLES::

            sage: from sage.doctest.parsing import MarkedOutput, SageOutputChecker
            sage: import doctest
            sage: optflag = doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS
            sage: rndstr = MarkedOutput("I'm wrong!").update(random=True)
            sage: tentol = MarkedOutput("10.0").update(tol=.1)
            sage: tenabs = MarkedOutput("10.0").update(abs_tol=.1)
            sage: tenrel = MarkedOutput("10.0").update(rel_tol=.1)
            sage: zerotol = MarkedOutput("0.0").update(tol=.1)
            sage: zeroabs = MarkedOutput("0.0").update(abs_tol=.1)
            sage: zerorel = MarkedOutput("0.0").update(rel_tol=.1)
            sage: zero = "0.0"
            sage: nf = "9.5"
            sage: ten = "10.05"
            sage: eps = "-0.05"
            sage: OC = SageOutputChecker()

        ::

            sage: OC.check_output(rndstr,nf,optflag)
            True

            sage: OC.check_output(tentol,nf,optflag)
            True
            sage: OC.check_output(tentol,ten,optflag)
            True
            sage: OC.check_output(tentol,zero,optflag)
            False

            sage: OC.check_output(tenabs,nf,optflag)
            False
            sage: OC.check_output(tenabs,ten,optflag)
            True
            sage: OC.check_output(tenabs,zero,optflag)
            False

            sage: OC.check_output(tenrel,nf,optflag)
            True
            sage: OC.check_output(tenrel,ten,optflag)
            True
            sage: OC.check_output(tenrel,zero,optflag)
            False

            sage: OC.check_output(zerotol,zero,optflag)
            True
            sage: OC.check_output(zerotol,eps,optflag)
            True
            sage: OC.check_output(zerotol,ten,optflag)
            False

            sage: OC.check_output(zeroabs,zero,optflag)
            True
            sage: OC.check_output(zeroabs,eps,optflag)
            True
            sage: OC.check_output(zeroabs,ten,optflag)
            False

            sage: OC.check_output(zerorel,zero,optflag)
            True
            sage: OC.check_output(zerorel,eps,optflag)
            False
            sage: OC.check_output(zerorel,ten,optflag)
            False

        More explicit tolerance checks::

            sage: _ = x  # rel tol 1e10                                                 # needs sage.symbolic
            sage: raise RuntimeError   # rel tol 1e10
            Traceback (most recent call last):
            ...
            RuntimeError
            sage: 1  # abs tol 2
            -0.5
            sage: print("0.9999")    # rel tol 1e-4
            1.0
            sage: print("1.00001")   # abs tol 1e-5
            1.0
            sage: 0  # rel tol 1
            1

        Abs tol checks over the complex domain::

            sage: [1, -1.3, -1.5 + 0.1*I, 0.5 - 0.1*I, -1.5*I]  # abs tol 1.0
            [1, -1, -1, 1, -I]

        Spaces before numbers or between the sign and number are ignored::

            sage: print("[ - 1, 2]")  # abs tol 1e-10
            [-1,2]

        Tolerance on Python 3 for string results with unicode prefix::

            sage: a = 'Cyrano'; a
            'Cyrano'
            sage: b = ['Fermat', 'Euler']; b
            ['Fermat',  'Euler']
            sage: c = 'you'; c
            'you'

        This illustrates that :issue:`33588` is fixed::

            sage: from sage.doctest.parsing import SageOutputChecker, SageDocTestParser
            sage: import doctest
            sage: optflag = doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS
            sage: DTP = SageDocTestParser(('sage','magma','guava'))
            sage: OC = SageOutputChecker()
            sage: example = "sage: 1.3090169943749475 # tol 1e-8\n1.3090169943749475"
            sage: ex = DTP.parse(example)[1]
            sage: OC.check_output(ex.want, '1.3090169943749475', optflag)
            True
            sage: OC.check_output(ex.want, 'ANYTHING1.3090169943749475', optflag)
            False
            sage: OC.check_output(ex.want, 'Long-step dual simplex will be used\n1.3090169943749475', optflag)
            True
        """
        got = self.human_readable_escape_sequences(got)
        try:
            if isinstance(want, MarkedOutput):
                if want.random:
                    return True
                elif want.tol or want.rel_tol:
                    want, got = check_tolerance_real_domain(want, got)
                elif want.abs_tol:
                    want, got = check_tolerance_complex_domain(want, got)
        except ToleranceExceededError:
            return False

        if doctest.OutputChecker.check_output(self, want, got, optionflags):
            return True
        else:
            # Last resort: try to fix-up the got string removing few typical warnings
            did_fixup, want, got = self.do_fixup(want, got)
            if did_fixup:
                return doctest.OutputChecker.check_output(self, want, got, optionflags)
            else:
                return False

    def do_fixup(self, want, got):
        r"""
        Perform few changes to the strings ``want`` and ``got``.

        For example, remove warnings to be ignored.

        INPUT:

        - ``want`` -- string or :class:`MarkedOutput`
        - ``got`` -- string

        OUTPUT: a tuple:

        - boolean, ``True`` when some fixup were performed and ``False`` otherwise
        - string, edited wanted string
        - string, edited got string

        .. NOTE::

            Currently, the code only possibly changes the string ``got``
            while keeping ``want`` invariant. We keep open the possibility
            of adding a regular expression which would also change the
            ``want`` string. This is why ``want`` is an input and an output
            of the method even if currently kept invariant.

        EXAMPLES::

            sage: from sage.doctest.parsing import SageOutputChecker
            sage: OC = SageOutputChecker()
            sage: OC.do_fixup('1.3090169943749475','1.3090169943749475')
            (False, '1.3090169943749475', '1.3090169943749475')
            sage: OC.do_fixup('1.3090169943749475','ANYTHING1.3090169943749475')
            (False, '1.3090169943749475', 'ANYTHING1.3090169943749475')
            sage: OC.do_fixup('1.3090169943749475','Long-step dual simplex will be used\n1.3090169943749475')
            (True, '1.3090169943749475', '\n1.3090169943749475')

        When ``want`` is an instance of class :class:`MarkedOutput`::

            sage: from sage.doctest.parsing import SageOutputChecker, SageDocTestParser
            sage: import doctest
            sage: optflag = doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS
            sage: DTP = SageDocTestParser(('sage','magma','guava'))
            sage: OC = SageOutputChecker()
            sage: example = "sage: 1.3090169943749475\n1.3090169943749475"
            sage: ex = DTP.parse(example)[1]
            sage: ex.want
            '1.3090169943749475\n'
            sage: OC.do_fixup(ex.want,'1.3090169943749475')
            (False, '1.3090169943749475\n', '1.3090169943749475')
            sage: OC.do_fixup(ex.want,'ANYTHING1.3090169943749475')
            (False, '1.3090169943749475\n', 'ANYTHING1.3090169943749475')
            sage: OC.do_fixup(ex.want,'Long-step dual simplex will be used\n1.3090169943749475')
            (True, '1.3090169943749475\n', '\n1.3090169943749475')
        """
        did_fixup = False

        # The conditions in the below `if` are simple fast test on the expected
        # and/or actual output to determine if a fixup should be applied.

        if "Long-step" in got:
            # Version 4.65 of glpk prints the warning "Long-step dual
            # simplex will be used" frequently. When Sage uses a system
            # installation of glpk which has not been patched, we need to
            # ignore that message. See :issue:`29317`.
            glpk_simplex_warning_regex = re.compile(r'(Long-step dual simplex will be used)')
            got = glpk_simplex_warning_regex.sub('', got)
            did_fixup = True

        if "chained fixups" in got:
            # :issue:`34533` -- suppress warning on OS X 12.6 about chained fixups
            chained_fixup_warning_regex = re.compile(r'ld: warning: -undefined dynamic_lookup may not work with chained fixups')
            got = chained_fixup_warning_regex.sub('', got)
            did_fixup = True

        if "newer macOS version" in got:
            # :issue:`34741` -- suppress warning arising after
            # upgrading from macOS 12.X to 13.X.
            newer_macOS_version_regex = re.compile(r'.*dylib \(.*\) was built for newer macOS version \(.*\) than being linked \(.*\)')
            got = newer_macOS_version_regex.sub('', got)
            did_fixup = True

        if "insufficient permissions" in got:
            sympow_cache_warning_regex = re.compile(r'\*\*WARNING\*\* /var/cache/sympow/datafiles/le64 yields insufficient permissions')
            got = sympow_cache_warning_regex.sub('', got)
            did_fixup = True

        if "dylib" in got:
            # :issue:`31204` -- suppress warning about ld and OS version for
            # dylib files.
            ld_warning_regex = re.compile(r'^.*dylib.*was built for newer macOS version.*than being linked.*')
            got = ld_warning_regex.sub('', got)
            did_fixup = True

        if "pie being ignored" in got:
            # :issue:`30845` -- suppress warning on conda about ld
            ld_pie_warning_regex = re.compile(r'ld: warning: -pie being ignored. It is only used when linking a main executable')
            got = ld_pie_warning_regex.sub('', got)
            did_fixup = True

        if "R[write to console]" in got:
            # Supress R warnings
            r_warning_regex = re.compile(r'R\[write to console\]:.*')
            got = r_warning_regex.sub('', got)
            did_fixup = True

        if "Overriding pythran description" in got:
            # Some signatures changed in numpy-1.25.x that may yet be
            # reverted, but which pythran would otherwise warn about.
            # Pythran has a special case for numpy.random that hides
            # the warning -- I guess until we know if the changes will
            # be reverted -- but only in v0.14.0 of pythran. Ignoring
            # This warning allows us to support older pythran with e.g.
            # numpy-1.25.2.
            pythran_numpy_warning_regex = re.compile(r'WARNING: Overriding pythran description with argspec information for: numpy\.random\.[a-z_]+')
            got = pythran_numpy_warning_regex.sub('', got)
            did_fixup = True

        if "ld_classic is deprecated" in got:
            ld_classic_warning_regex = re.compile(r'ld: warning: -ld_classic is deprecated and will be removed in a future release')
            got = ld_classic_warning_regex.sub('', got)
            did_fixup = True

        return did_fixup, want, got

    def output_difference(self, example, got, optionflags):
        r"""
        Report on the differences between the desired result and what
        was actually obtained.

        If ``want`` is a :class:`MarkedOutput` instance, takes into account the desired tolerance.

        INPUT:

        - ``example`` -- a :class:`doctest.Example` instance
        - ``got`` -- string
        - ``optionflags`` -- integer; passed down to :class:`doctest.OutputChecker`

        OUTPUT: string, describing how ``got`` fails to match ``example.want``

        EXAMPLES::

            sage: from sage.doctest.parsing import MarkedOutput, SageOutputChecker
            sage: import doctest
            sage: optflag = doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS
            sage: tentol = doctest.Example('',MarkedOutput("10.0\n").update(tol=.1))
            sage: tenabs = doctest.Example('',MarkedOutput("10.0\n").update(abs_tol=.1))
            sage: tenrel = doctest.Example('',MarkedOutput("10.0\n").update(rel_tol=.1))
            sage: zerotol = doctest.Example('',MarkedOutput("0.0\n").update(tol=.1))
            sage: zeroabs = doctest.Example('',MarkedOutput("0.0\n").update(abs_tol=.1))
            sage: zerorel = doctest.Example('',MarkedOutput("0.0\n").update(rel_tol=.1))
            sage: tlist = doctest.Example('',MarkedOutput("[10.0, 10.0, 10.0, 10.0, 10.0, 10.0]\n").update(abs_tol=0.987))
            sage: zero = "0.0"
            sage: nf = "9.5"
            sage: ten = "10.05"
            sage: eps = "-0.05"
            sage: L = "[9.9, 8.7, 10.3, 11.2, 10.8, 10.0]"
            sage: OC = SageOutputChecker()

        ::

            sage: print(OC.output_difference(tenabs,nf,optflag))
            Expected:
                10.0
            Got:
                9.5
            Tolerance exceeded:
                10.0 vs 9.5, tolerance 5e-1 > 1e-1

            sage: print(OC.output_difference(tentol,zero,optflag))
            Expected:
                10.0
            Got:
                0.0
            Tolerance exceeded:
                10.0 vs 0.0, tolerance 1e0 > 1e-1

            sage: print(OC.output_difference(tentol,eps,optflag))
            Expected:
                10.0
            Got:
                -0.05
            Tolerance exceeded:
                10.0 vs -0.05, tolerance 2e0 > 1e-1

            sage: print(OC.output_difference(tlist,L,optflag))
            Expected:
                [10.0, 10.0, 10.0, 10.0, 10.0, 10.0]
            Got:
                [9.9, 8.7, 10.3, 11.2, 10.8, 10.0]
            Tolerance exceeded in 2 of 6:
                10.0 vs 8.7, tolerance 2e0 > 9.87e-1
                10.0 vs 11.2, tolerance 2e0 > 9.87e-1

        TESTS::

            sage: print(OC.output_difference(tenabs,zero,optflag))
            Expected:
                10.0
            Got:
                0.0
            Tolerance exceeded:
                10.0 vs 0.0, tolerance 1e1 > 1e-1

            sage: print(OC.output_difference(tenrel,zero,optflag))
            Expected:
                10.0
            Got:
                0.0
            Tolerance exceeded:
                10.0 vs 0.0, tolerance 1e0 > 1e-1

            sage: print(OC.output_difference(tenrel,eps,optflag))
            Expected:
                10.0
            Got:
                -0.05
            Tolerance exceeded:
                10.0 vs -0.05, tolerance 2e0 > 1e-1

            sage: print(OC.output_difference(zerotol,ten,optflag))
            Expected:
                0.0
            Got:
                10.05
            Tolerance exceeded:
                0.0 vs 10.05, tolerance 2e1 > 1e-1

            sage: print(OC.output_difference(zeroabs,ten,optflag))
            Expected:
                0.0
            Got:
                10.05
            Tolerance exceeded:
                0.0 vs 10.05, tolerance 2e1 > 1e-1

            sage: print(OC.output_difference(zerorel,eps,optflag))
            Expected:
                0.0
            Got:
                -0.05
            Tolerance exceeded:
                0.0 vs -0.05, tolerance +infinity > 1e-1

            sage: print(OC.output_difference(zerorel,ten,optflag))
            Expected:
                0.0
            Got:
                10.05
            Tolerance exceeded:
                0.0 vs 10.05, tolerance +infinity > 1e-1
        """
        got = self.human_readable_escape_sequences(got)
        want = example.want
        diff = doctest.OutputChecker.output_difference(self, example, got, optionflags)
        if isinstance(want, MarkedOutput) and (want.tol or want.abs_tol or want.rel_tol):
            if diff[-1] != "\n":
                diff += "\n"
            want_str = [g[0] for g in float_regex.findall(want)]
            got_str = [g[0] for g in float_regex.findall(got)]
            if len(want_str) == len(got_str):
                failures = []

                def fail(x, y, actual, desired):
                    failstr = "    {} vs {}, tolerance {} > {}".format(x, y,
                        RIFtol(actual).upper().str(digits=1, no_sci=False),
                        RIFtol(desired).center().str(digits=15, skip_zeroes=True, no_sci=False)
                    )
                    failures.append(failstr)

                for wstr, gstr in zip(want_str, got_str):
                    w = RIFtol(wstr)
                    g = RIFtol(gstr)
                    if not g.overlaps(add_tolerance(w, want)):
                        if want.tol:
                            if not w:
                                fail(wstr, gstr, abs(g), want.tol)
                            else:
                                fail(wstr, gstr, abs(1 - g / w), want.tol)
                        elif want.abs_tol:
                            fail(wstr, gstr, abs(g - w), want.abs_tol)
                        else:
                            fail(wstr, gstr, abs(1 - g / w), want.rel_tol)

                if failures:
                    if len(want_str) == 1:
                        diff += "Tolerance exceeded:\n"
                    else:
                        diff += "Tolerance exceeded in %s of %s:\n" % (len(failures), len(want_str))
                    diff += "\n".join(failures) + "\n"
            elif "..." in want:
                diff += "Note: combining tolerance (# tol) with ellipsis (...) is not supported\n"
        return diff



# sage_setup: distribution = sagemath-repl
r"""
Keep track of attached files

TESTS::

    sage: attach('http://wstein.org/loadtest.py')
    Traceback (most recent call last):
    ...
    NotImplementedError: you cannot attach a URL

Check that no file clutter is produced::

    sage: dir = tmp_dir()
    sage: src = os.path.join(dir, 'foobar.sage')
    sage: with open(src, 'w') as f:
    ....:     _ = f.write('print("<output from attached file>")\n')
    sage: attach(src)
    <output from attached file>
    sage: os.listdir(dir)
    ['foobar.sage']
    sage: detach(src)

In debug mode backtraces contain code snippets. We need to manually
print the traceback because the python doctest module has special
support for exceptions and does not match them
character-by-character::

    sage: import traceback
    sage: with open(src, 'w') as f:
    ....:     _ = f.write('# first line\n')
    ....:     _ = f.write('# second line\n')
    ....:     _ = f.write('raise ValueError("third")   # this should appear in the source snippet\n')
    ....:     _ = f.write('# fourth line\n')

    sage: load_attach_mode(attach_debug=False)
    sage: try:
    ....:     attach(src)
    ....: except Exception:
    ....:     traceback.print_exc(file=sys.stdout)
    Traceback (most recent call last):
    ...
        exec(preparse_file(f.read()) + "\n", globals)
      File "<string>", line 3, in <module>
    ValueError: third
    sage: detach(src)

    sage: load_attach_mode(attach_debug=True)
    sage: try:
    ....:     attach(src)
    ....: except Exception:
    ....:     traceback.print_exc(file=sys.stdout)
    Traceback (most recent call last):
    ...
        exec(code, globals)
      File ".../foobar...sage.py", line ..., in <module>
        raise ValueError("third")   # this should appear in the source snippet...
    ValueError: third
    sage: detach(src)
"""

# ****************************************************************************
#       Copyright (C) 2013 Volker Braun <vbraun.name@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************
from __future__ import annotations

import os
import time
from pathlib import Path
from IPython.core.getipython import get_ipython

from sage.repl.load import load, load_wrap
import sage.repl.inputhook
import sage.env

# The attached files as a dict of {Path object:mtime}
attached = {}


load_debug_mode = False
attach_debug_mode = True


def load_attach_mode(load_debug=None, attach_debug=None):
    """
    Get or modify the current debug mode for the behavior of
    :func:`load` and :func:`attach` on ``.sage`` files.

    In debug mode, loaded or attached ``.sage`` files are preparsed
    through a file to make their tracebacks more informative. If not
    in debug mode, then ``.sage`` files are preparsed in memory only
    for performance.

    At startup, debug mode is ``True`` for attaching and ``False``
    for loading.

    .. NOTE::

        This function should really be deprecated and code executed
        from memory should raise proper tracebacks.

    INPUT:

    - ``load_debug`` -- boolean or ``None`` (default); if not
      ``None``, then set a new value for the debug mode for loading
      files

    - ``attach_debug`` -- boolean or ``None`` (default); same as
      ``load_debug``, but for attaching files

    OUTPUT:

    If all input values are ``None``, returns a tuple giving the
    current modes for loading and attaching.

    EXAMPLES::

        sage: load_attach_mode()
        (False, True)
        sage: load_attach_mode(attach_debug=False)
        sage: load_attach_mode()
        (False, False)
        sage: load_attach_mode(load_debug=True)
        sage: load_attach_mode()
        (True, False)
        sage: load_attach_mode(load_debug=False, attach_debug=True)
    """
    global load_debug_mode, attach_debug_mode
    if load_debug is None and attach_debug is None:
        return (load_debug_mode, attach_debug_mode)
    if load_debug is not None:
        load_debug_mode = load_debug
    if attach_debug is not None:
        attach_debug_mode = attach_debug


search_paths: list[Path] = []


def load_attach_path(path=None, replace=False):
    """
    Get or modify the current search path for :func:`load` and
    :func:`attach`.

    INPUT:

    - ``path`` -- string or list of strings (default: ``None``);
      path(s) to append to or replace the current path

    - ``replace`` -- boolean (default: ``False``); if ``path`` is not
      ``None``, whether to *replace* the search path instead of
      *appending* to it

    OUTPUT: none or a *reference* to the current search paths

    EXAMPLES:

    First, we extend the example given in :func:`load`'s docstring::

        sage: sage.repl.attach.reset(); reset_load_attach_path()
        sage: load_attach_path()
        [PosixPath('.')]
        sage: t_dir = tmp_dir()
        sage: fullpath = os.path.join(t_dir, 'test.py')
        sage: with open(fullpath, 'w') as f:
        ....:     _ = f.write("print(37 * 3)")

    We put a new, empty directory on the attach path for testing
    (otherwise this will load ``test.py`` from the current working
    directory if that happens to exist)::

        sage: import tempfile
        sage: with tempfile.TemporaryDirectory() as d:
        ....:     load_attach_path(d, replace=True)
        ....:     attach('test.py')
        Traceback (most recent call last):
        ...
        OSError: did not find file 'test.py' to load or attach
        sage: load_attach_path(t_dir)
        sage: attach('test.py')
        111
        sage: af = attached_files(); len(af)
        1
        sage: af == [fullpath]
        True
        sage: from pathlib import Path
        sage: sage.repl.attach.reset(); reset_load_attach_path()
        sage: load_attach_path() == [Path('.')]
        True
        sage: with tempfile.TemporaryDirectory() as d:
        ....:     load_attach_path(d, replace=True)
        ....:     load('test.py')
        Traceback (most recent call last):
        ...
        OSError: did not find file 'test.py' to load or attach

    The function returns a reference to the path list::

        sage: reset_load_attach_path(); load_attach_path()
        [PosixPath('.')]
        sage: load_attach_path('/path/to/my/sage/scripts'); load_attach_path()
        [PosixPath('.'), PosixPath('/path/to/my/sage/scripts')]
        sage: load_attach_path(['good', 'bad', 'ugly'], replace=True)
        sage: load_attach_path()
        [PosixPath('good'), PosixPath('bad'), PosixPath('ugly')]
        sage: p = load_attach_path(); p.pop()
        PosixPath('ugly')
        sage: p[0] = 'weird'; load_attach_path()
        ['weird', PosixPath('bad')]
        sage: reset_load_attach_path(); load_attach_path()
        [PosixPath('.')]
    """
    global search_paths
    if path is None:
        return search_paths

    if not isinstance(path, list):
        path = [Path(path)]
    if replace:
        search_paths = [Path(p) for p in path]
    else:
        for p in path:
            if not p:
                continue
            as_path = Path(p)
            if as_path not in search_paths:
                search_paths.append(as_path)


def reset_load_attach_path():
    """
    Reset the current search path for :func:`load` and :func:`attach`.

    The default path is ``'.'`` plus any paths specified in the
    environment variable ``SAGE_LOAD_ATTACH_PATH``.

    EXAMPLES::

        sage: load_attach_path()
        [PosixPath('.')]
        sage: t_dir = tmp_dir()
        sage: load_attach_path(t_dir)
        sage: from pathlib import Path
        sage: Path(t_dir) in load_attach_path()
        True
        sage: reset_load_attach_path(); load_attach_path()
        [PosixPath('.')]

    At startup, Sage adds colon-separated paths in the environment
    variable ``SAGE_LOAD_ATTACH_PATH``::

        sage: reset_load_attach_path(); load_attach_path()
        [PosixPath('.')]
        sage: os.environ['SAGE_LOAD_ATTACH_PATH'] = '/veni/vidi:vici:'
        sage: from importlib import reload
        sage: reload(sage.repl.attach)    # Simulate startup
        <module 'sage.repl.attach' from '...'>
        sage: load_attach_path()
        [PosixPath('.'), PosixPath('/veni/vidi'), PosixPath('vici')]
        sage: del os.environ['SAGE_LOAD_ATTACH_PATH']
        sage: reload(sage.repl.preparse)    # Simulate startup
        <module 'sage.repl.preparse' from '...'>
        sage: reset_load_attach_path(); load_attach_path()
        [PosixPath('.')]
    """
    global search_paths
    search_paths = [Path()]
    for path in os.environ.get('SAGE_LOAD_ATTACH_PATH', '').split(':'):
        load_attach_path(path=path)


# Set up the initial search path for loading and attaching files.  A
# user can modify the path with the function load_attach_path.
reset_load_attach_path()


def attach(*files):
    """
    Attach a file or files to a running instance of Sage and also load
    that file.

    .. NOTE::

        Attaching files uses the Python inputhook, which will conflict
        with other inputhook users. This generally includes GUI main loop
        integrations, for example tkinter. So you can only use tkinter or
        attach, but not both at the same time.

    INPUT:

    - ``files`` -- list of filenames (strings) to attach

    OUTPUT:

    Each file is read in and added to an internal list of watched files.
    The meaning of reading in a file depends on the file type:

    - ``.py`` files are read in with no preparsing (so, e.g., ``2^3`` is 2
      bit-xor 3);

    - ``.sage`` files are preparsed, then the result is read in;

    - ``.pyx`` files are *not* preparsed, but rather are compiled to a
      module ``m`` and then ``from m import *`` is executed.

    The contents of the file are then loaded, which means they are read
    into the running Sage session. For example, if ``foo.sage`` contains
    ``x=5``, after attaching ``foo.sage`` the variable ``x`` will be set
    to 5. Moreover, any time you change ``foo.sage``, before you execute
    a command, the attached file will be re-read automatically (with no
    intervention on your part).

    .. SEEALSO::

        :meth:`~sage.repl.load.load` is the same as :func:`attach`, but
        does not automatically reload a file when it changes.

    EXAMPLES:

    You attach a file, e.g., ``foo.sage`` or ``foo.py`` or
    ``foo.pyx``, to a running Sage session by typing::

        sage: attach('foo.sage')  # not tested

    Here we test attaching multiple files at once::

        sage: sage.repl.attach.reset()
        sage: t1 = tmp_filename(ext='.py')
        sage: with open(t1,'w') as f: _ = f.write("print('hello world')")
        sage: t2 = tmp_filename(ext='.py')
        sage: with open(t2,'w') as f: _ = f.write("print('hi there xxx')")
        sage: attach(t1, t2)
        hello world
        hi there xxx
        sage: af = attached_files(); len(af)
        2
        sage: t1 in af and t2 in af
        True

    .. SEEALSO::

        - :meth:`attached_files` returns a list of
          all currently attached files.

        - :meth:`detach` instructs Sage to remove a
          file from the internal list of watched files.

        - :meth:`load_attach_path` allows you to
          get or modify the current search path for loading and attaching
          files.
    """
    try:
        ipy = get_ipython()
    except NameError:
        ipy = None
    global attached
    for filename in files:
        if ipy:
            code = load_wrap(filename, attach=True)
            ipy.run_cell(code)
        else:
            load(filename, globals(), attach=True)


def add_attached_file(filename):
    """
    Add to the list of attached files.

    This is a callback to be used from
    :func:`~sage.repl.load.load` after evaluating the attached
    file the first time.

    INPUT:

    - ``filename`` -- string (the fully qualified file name)
      or :class:`Path` object

    EXAMPLES::

        sage: import sage.repl.attach as af
        sage: af.reset()
        sage: t = tmp_filename(ext='.py')
        sage: af.add_attached_file(t)
        sage: af.attached_files()
        ['/.../tmp_....py']
        sage: af.detach(t)
        sage: af.attached_files()
        []
    """
    sage.repl.inputhook.install()
    fpath = Path(filename).absolute()
    attached[fpath] = fpath.stat().st_mtime


def attached_files() -> list:
    """
    Return a list of all files attached to the current session with
    :meth:`attach`.

    OUTPUT: the filenames in a sorted list of strings

    EXAMPLES::

        sage: sage.repl.attach.reset()
        sage: t = tmp_filename(ext='.py')
        sage: with open(t,'w') as f: _ = f.write("print('hello world')")
        sage: attach(t)
        hello world
        sage: af = attached_files(); af
        ['/....py']
        sage: af == [t]
        True
    """
    global attached
    return sorted(str(f) for f in attached)


def detach(filename):
    """
    Detach a file.

    This is the counterpart to :meth:`attach`.

    INPUT:

    - ``filename`` -- string, a list of strings or a tuple of strings
      or a :class:`Path`, a list of :class:`Path` or a tuple of :class:`Path`

    EXAMPLES::

        sage: sage.repl.attach.reset()
        sage: t = tmp_filename(ext='.py')
        sage: with open(t,'w') as f: _ = f.write("print('hello world')")
        sage: attach(t)
        hello world
        sage: af = attached_files(); len(af)
        1
        sage: af == [t]
        True
        sage: detach(t)
        sage: attached_files()
        []

        sage: sage.repl.attach.reset(); reset_load_attach_path()
        sage: load_attach_path()
        [PosixPath('.')]
        sage: t_dir = tmp_dir()
        sage: fullpath = os.path.join(t_dir, 'test.py')
        sage: with open(fullpath, 'w') as f: _ = f.write("print(37 * 3)")
        sage: load_attach_path(t_dir, replace=True)
        sage: attach('test.py')
        111
        sage: af = attached_files(); len(af)
        1
        sage: af == [os.path.normpath(fullpath)]
        True
        sage: detach('test.py')
        sage: attached_files()
        []
        sage: attach('test.py')
        111
        sage: fullpath = os.path.join(t_dir, 'test2.py')
        sage: with open(fullpath, 'w') as f: _ = f.write("print(3)")
        sage: attach('test2.py')
        3
        sage: detach(attached_files())
        sage: attached_files()
        []

    TESTS::

        sage: detach('/dev/null/foobar.sage')
        Traceback (most recent call last):
        ...
        ValueError: file '/dev/null/foobar.sage' is not attached, see attached_files()
    """
    if isinstance(filename, str):
        filelist = [Path(filename)]
    else:
        filelist = [Path(x) for x in filename]

    global attached
    for filename in filelist:
        fpath = filename.expanduser()
        if not fpath.is_absolute():
            for path in load_attach_path():
                fpath = path.expanduser() / filename
                fpath = fpath.absolute()
                if fpath in attached:
                    break
        abs_fpath = fpath.absolute()
        if fpath in attached:
            attached.pop(fpath)
        elif abs_fpath in attached:
            attached.pop(abs_fpath)
        else:
            raise ValueError("file '{0}' is not attached, see attached_files()".format(filename))
    if not attached:
        sage.repl.inputhook.uninstall()


def reset():
    """
    Remove all the attached files from the list of attached files.

    EXAMPLES::

        sage: sage.repl.attach.reset()
        sage: t = tmp_filename(ext='.py')
        sage: with open(t,'w') as f: _ = f.write("print('hello world')")
        sage: attach(t)
        hello world
        sage: af = attached_files(); len(af)
        1
        sage: af == [t]
        True
        sage: sage.repl.attach.reset()
        sage: attached_files()
        []
    """
    global attached
    attached = {}


def modified_file_iterator():
    """
    Iterate over the changed files.

    As a side effect the stored time stamps are updated with the
    actual time stamps. So if you iterate over the attached files in
    order to reload them and you hit an error then the subsequent
    files are not marked as read.

    Files that are in the process of being saved are excluded.

    EXAMPLES::

        sage: sage.repl.attach.reset()
        sage: t = tmp_filename(ext='.py')
        sage: attach(t)
        sage: from sage.repl.attach import modified_file_iterator
        sage: list(modified_file_iterator())
        []
        sage: sleep(1)   # filesystem mtime granularity
        sage: with open(t, 'w') as f: _ = f.write('1')
        sage: list(modified_file_iterator())
        [(PosixPath('/.../tmp_....py'), time.struct_time(...))]
    """
    global attached
    modified = {}
    for filename in list(attached):
        old_tm = attached[filename]
        if not filename.exists():
            print('### detaching file {0} because it does not exist (deleted?) ###'.format(filename))
            detach(filename)
            continue
        new_tm = filename.stat().st_mtime
        if new_tm > old_tm:
            modified[filename] = new_tm

    if not modified:
        return
    time.sleep(0.1)  # sleep 100ms to give the editor time to finish saving

    for filename in list(modified):
        old_tm = modified[filename]
        new_tm = filename.stat().st_mtime
        if new_tm == old_tm:
            # file was modified but did not change in the last 100ms
            attached[filename] = new_tm
            yield filename, time.gmtime(new_tm)


def reload_attached_files_if_modified():
    r"""
    Reload attached files that have been modified.

    This is the internal implementation of the attach mechanism.

    EXAMPLES::

        sage: sage.repl.attach.reset()
        sage: from sage.repl.interpreter import get_test_shell
        sage: shell = get_test_shell()
        sage: tmp = tmp_filename(ext='.py')
        sage: with open(tmp, 'w') as f: _ = f.write('a = 2\n')
        sage: shell.run_cell('attach({0})'.format(repr(tmp)))
        sage: shell.run_cell('a')
        2
        sage: sleep(1)   # filesystem mtime granularity
        sage: with open(tmp, 'w') as f: _ = f.write('a = 3\n')

    Note that the doctests are never really at the command prompt
    where the automatic reload is triggered. So we have to do it
    manually::

        sage: shell.run_cell('from sage.repl.attach import reload_attached_files_if_modified')
        sage: shell.run_cell('reload_attached_files_if_modified()')
        ### reloading attached file tmp_....py modified at ... ###

        sage: shell.run_cell('a')
        3
        sage: shell.run_cell('detach({0})'.format(repr(tmp)))
        sage: shell.run_cell('attached_files()')
        []
        sage: shell.quit()
    """
    ip = get_ipython()
    for filename, mtime in modified_file_iterator():
        basename = filename.name
        timestr = time.strftime('%T', mtime)
        notice = '### reloading attached file {0} modified at {1} ###'.format(basename, timestr)
        if ip:
            print(notice)
            code = load_wrap(filename, attach=True)
            ip.run_cell(code)
        else:
            print(notice)
            load(filename, globals(), attach=True)



# sage_setup: distribution = sagemath-repl
"""
Fortran compiler
"""

import importlib
import os
import shutil
import subprocess
import sys

from sage.misc.temporary_file import tmp_dir


def _import_module_from_path(name, path=None):
    """
    Import the module named ``name`` by searching the given path entries (or
    `sys.path` by default).

    Returns a fully executed module object without inserting that module into
    `sys.modules`.

    EXAMPLES::

        sage: from sage.misc.inline_fortran import _import_module_from_path
        sage: modname = '___test__import_module_from_path'
        sage: tmpdir = tmp_dir()
        sage: filename = os.path.join(tmpdir, modname + '.py')
        sage: with open(filename, 'w') as fobj:
        ....:     _ = fobj.write('foo = "bar"')
        sage: mod = _import_module_from_path(modname)
        Traceback (most recent call last):
        ...
        ImportError: No module named ___test__import_module_from_path
        sage: mod = _import_module_from_path('DoEsNoTeXiSt', path=[tmpdir])
        Traceback (most recent call last):
        ...
        ImportError: No module named DoEsNoTeXiSt
        sage: mod = _import_module_from_path(modname, path=[tmpdir])
        sage: mod.foo
        'bar'
    """

    if path is None:
        path = sys.path

    return _import_module_from_path_impl(name, path)


def _import_module_from_path_impl(name, path):
    """Implement ``_import_module_from_path`` for Python 3.4+."""

    # This is remarkably tricky to do right, considering that the new
    # importlib is supposed to make direct interaction with the import
    # system easier.  I blame the ModuleSpec stuff...
    finder = importlib.machinery.PathFinder()
    spec = finder.find_spec(name, path=path)
    if spec is None:
        raise ImportError('No module named {}'.format(name))
    mod = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(mod)
    return mod


class InlineFortran:
    def __init__(self, globals=None):
        # globals=None means: use user globals from REPL
        self.globs = globals
        self.library_paths = []
        self.libraries = []
        self.verbose = False

    def __repr__(self):
        return "Interface to Fortran compiler"

    def __call__(self, *args, **kwds):
        return self.eval(*args, **kwds)

    def eval(self, x, globals=None, locals=None):
        """
        Compile fortran code ``x`` and adds the functions in it to ``globals``.

        INPUT:

        - ``x`` -- fortran code

        - ``globals`` -- dictionary to which to add the functions from the
          fortran module

        - ``locals`` -- ignored

        EXAMPLES::

            sage: # needs numpy
            sage: code = '''
            ....: C FILE: FIB1.F
            ....:       SUBROUTINE FIB(A,N)
            ....: C
            ....: C     CALCULATE FIRST N FIBONACCI NUMBERS
            ....: C
            ....:       INTEGER N
            ....:       REAL*8 A(N)
            ....:       DO I=1,N
            ....:          IF (I.EQ.1) THEN
            ....:             A(I) = 0.0D0
            ....:          ELSEIF (I.EQ.2) THEN
            ....:             A(I) = 1.0D0
            ....:          ELSE
            ....:             A(I) = A(I-1) + A(I-2)
            ....:          ENDIF
            ....:       ENDDO
            ....:       END
            ....: C END FILE FIB1.F
            ....: '''
            sage: fortran(code, globals())
            sage: import numpy
            sage: a = numpy.array(range(10), dtype=float)
            sage: fib(a, 10)
            sage: a
            array([  0.,   1.,   1.,   2.,   3.,   5.,   8.,  13.,  21.,  34.])

        TESTS::

            sage: os.chdir(DOT_SAGE)
            sage: fortran.eval("SYNTAX ERROR !@#$")
            Traceback (most recent call last):
            ...
            RuntimeError: failed to compile Fortran code:...
            sage: os.getcwd() == os.path.realpath(DOT_SAGE)
            True
        """
        if globals is None:
            globals = self.globs
            if globals is None:
                from sage.repl.user_globals import get_globals
                globals = get_globals()

        # Create everything in a temporary directory
        mytmpdir = tmp_dir()

        try:
            old_cwd = os.getcwd()
            os.chdir(mytmpdir)

            name = "fortran_module"  # Python module name
            # if the first line has !f90 as a comment, gfortran will
            # treat it as Fortran 90 code
            if x.startswith('!f90'):
                fortran_file = name + '.f90'
            else:
                fortran_file = name + '.f'

            s_lib_path = ['-L' + p for p in self.library_paths]
            s_lib = ['-l' + l for l in self.libraries]

            with open(fortran_file, 'w') as fobj:
                fobj.write(x)

            # This is basically the same as what f2py.compile() does, but we
            # do it manually here in order to customize running the subprocess
            # a bit more (in particular to capture stderr)
            cmd = [sys.executable, '-c', 'import numpy.f2py; numpy.f2py.main()']

            # What follows are the arguments to f2py itself (appended later
            # just for logical separation)
            cmd += ['-c', '-m', name, fortran_file, '--quiet',
                    '--f77exec=sage-inline-fortran',
                    '--f90exec=sage-inline-fortran'] + s_lib_path + s_lib

            try:
                out = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
            except subprocess.CalledProcessError as exc:
                raise RuntimeError(
                    "failed to compile Fortran code:\n{}".format(exc.output))

            # Note that f2py() doesn't raise an exception if it fails.
            # In that case, the import below will fail.
            try:
                mod = _import_module_from_path(name, [mytmpdir])
            except ImportError as exc:
                # Failed to import the module; include any output from building
                # the module (even though it was ostensibly successful) in case
                # it might help
                msg = "failed to load compiled Fortran code: {}".format(exc)
                if out:
                    msg += '\n' + out
                raise RuntimeError(msg)

            if self.verbose:
                print(out)
        finally:
            os.chdir(old_cwd)

            try:
                shutil.rmtree(mytmpdir)
            except OSError:
                # This can fail for example over NFS
                pass

        for k, x in mod.__dict__.items():
            if k[0] != '_':
                globals[k] = x

    def add_library(self, s):
        self.libraries.append(s)

    def add_library_path(self, s):
        self.library_paths.append(s)


# An instance
fortran = InlineFortran()



# sage_setup: distribution = sagemath-repl
r"""
Class inheritance graphs
"""
# ****************************************************************************
#  Copyright (C) 2007 William Stein <wstein@math.ucsd.edu>
#                2011 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
# *****************************************************************************

import inspect


def class_graph(top, depth=5, name_filter=None, classes=None, as_graph=True):
    """
    Return the class inheritance graph of a module, class, or object.

    INPUT:

    - ``top`` -- the module, class, or object to start with (e.g. ``sage``,
      ``Integer``, ``3``)
    - ``depth`` -- maximal recursion depth within submodules (default: 5)
    - ``name_filter`` -- e.g. 'sage.rings' to only consider classes in
      :mod:`sage.rings`
    - ``classes`` -- (optional) dictionary to be filled in (it is also returned)
    - ``as_graph`` -- boolean (default: ``True``)

    OUTPUT:

    An oriented graph, with class names as vertices, and an edge
    from each class to each of its bases.

    EXAMPLES:

    We construct the inheritance graph of the classes within a given module::

        sage: from sage.rings.polynomial.padics import polynomial_padic_capped_relative_dense, polynomial_padic_flat
        sage: G = class_graph(sage.rings.polynomial.padics); G
        Digraph on 6 vertices
        sage: G.vertices(sort=True)
        ['Polynomial',
         'Polynomial_generic_cdv',
         'Polynomial_generic_dense',
         'Polynomial_padic',
         'Polynomial_padic_capped_relative_dense',
         'Polynomial_padic_flat']
        sage: G.edges(sort=True, labels=False)
        [('Polynomial_padic', 'Polynomial'),
         ('Polynomial_padic_capped_relative_dense', 'Polynomial_generic_cdv'),
         ('Polynomial_padic_capped_relative_dense', 'Polynomial_padic'),
         ('Polynomial_padic_flat', 'Polynomial_generic_dense'),
         ('Polynomial_padic_flat', 'Polynomial_padic')]

    We construct the inheritance graph of a given class::

        sage: class_graph(Parent).edges(sort=True, labels=False)
        [('CategoryObject', 'SageObject'), ('Parent', 'CategoryObject'), ('SageObject', 'object')]

    We construct the inheritance graph of the class of an object::

        sage: class_graph([1,2,3]).edges(sort=True, labels=False)
        [('list', 'object')]

    .. warning:: the output of ``class_graph`` used to be a dictionary
       mapping each class name to the list of names of its bases. This
       can be emulated by setting the option ``as_graph`` to ``False``::

        sage: class_graph(sage.rings.polynomial.padics, depth=2, as_graph=False)
        {'Polynomial_padic': ['Polynomial'],
         'Polynomial_padic_capped_relative_dense': ['Polynomial_generic_cdv',
          'Polynomial_padic'],
         'Polynomial_padic_flat': ['Polynomial_generic_dense', 'Polynomial_padic']}

    .. NOTE:: the ``classes`` and ``as_graph`` options are mostly
       intended for internal recursive use.

    .. NOTE:: ``class_graph`` does not yet handle nested classes

    TESTS::

        sage: G = class_graph(sage.rings.polynomial.padics, depth=2); G
        Digraph on 6 vertices
    """
    # This function descends recursively down the submodules of the
    # top module (if ``top`` is a module) and then down the hierarchy
    # of classes. Along the way, the result is stored in the "global"
    # dictionary ``classes`` which associates to each class the list
    # of its bases.

    # Termination
    if depth < 0:
        return classes

    # (first recursive call)
    if classes is None:
        classes = {}

    # Build the list ``children`` of submodules (resp. base classes)
    # of ``top`` the function will recurse through
    if inspect.ismodule(top):
        if top.__name__.endswith('.all'):  # Ignore sage.rings.all and friends
            return classes
        if name_filter is None:
            name_filter = top.__name__
        children = [item for item in top.__dict__.values()
                    if inspect.ismodule(item) or inspect.isclass(item)]
        depth -= 1
    elif inspect.isclass(top):
        if name_filter is None:
            name_filter = ""
        if not top.__module__.startswith(name_filter):
            return classes
        children = top.__bases__
        classes[top.__name__] = [e.__name__ for e in children]
    else:  # top is a plain Python object; inspect its class
        children = [top.__class__]

    # Recurse
    for child in children:
        class_graph(child, depth=depth, name_filter=name_filter,
                    classes=classes, as_graph=False)

    # (first recursive call): construct the graph
    if as_graph:
        from sage.graphs.digraph import DiGraph
        return DiGraph(classes)
    else:
        return classes



# sage_setup: distribution = sagemath-repl
"""
Launch valgrind and run the memory leak tests


From the commandline, run

    sage -python -m sage.tests.memcheck.run_tests_in_valgrind

to launch valgrind and execute the memory leak tests. Requires valgrind
to be installed. Alternatively, run as a unit test:

    sage: from sage.tests.memcheck.run_tests_in_valgrind import run_tests_in_valgrind
    sage: run_tests_in_valgrind()    # optional - valgrind
"""

import subprocess


def run_tests_in_valgrind() -> None:
    """
    Run the sage.tests.memcheck.run_tests module inside valgrind
    """
    subprocess.check_call([
        'valgrind',
        '--suppressions=src/sage/ext_data/valgrind/valgrind-python.supp',
        '--show-possibly-lost=no',
        '--show-reachable=no',
        './venv/bin/python',
        '-m',
        'sage.tests.memcheck.run_tests'
    ])


if __name__ == '__main__':
    run_tests_in_valgrind()



# sage_setup: distribution = sagemath-repl
r"""
This file (./linsolve_doctest.sage) was *autogenerated* from ./linsolve.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./linsolve_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./linsolve.tex, line 294::

  sage: def cond_hilbert(n):
  ....:     A = matrix(QQ, [[1/(i+j-1) for j in [1..n]] for i in [1..n]])
  ....:     return A.norm(Infinity) * (A^-1).norm(Infinity)

Sage example in ./linsolve.tex, line 346::

  sage: def diff_hilbert(n):
  ....:     x = vector(QQ,[1 for i in range(0,n)])
  ....:     A = matrix(QQ, [[1/(i+j-1) for j in [1..n]] for i in [1..n]])
  ....:     y = A*x
  ....:     A[n-1,n-1] = (1/(2*n-1))*(1+1/(10^5))   # modifies the matrix
  ....:     s = A\y
  ....:     return max(abs(float(s[i]-x[i])) for i in range(0,n))

Sage example in ./linsolve.tex, line 396::

  sage: def hilbert_diff(n):
  ....:     j = var("j")
  ....:     f = lambda i: sum(1/(i+j-1),j,1,n)
  ....:     y = vector(RDF, [f(i+1) for i in range(0,n)])
  ....:     A = matrix(RDF, [[1/(i+j-1) for i in [1..n]] for j in [1..n]])
  ....:     x = A.solve_right(y)
  ....:     return max(abs(x[i]-1.0) for i in range(0,n))

Sage example in ./linsolve.tex, line 543::

  sage: n = 20; cost = (n+1)*factorial(n); cost
  51090942171709440000

Sage example in ./linsolve.tex, line 559::

  sage: v = 3*10^9
  sage: print("%3.3f" % float(cost/v/3600/24/365))
  540.028

Sage example in ./linsolve.tex, line 653::

  sage: A = matrix(RDF, [[-1,2],[3,4]])
  sage: b = vector(RDF, [2,3])
  sage: x = A.solve_right(b); x
  (-0.20000000000000018, 0.9000000000000001)

Sage example in ./linsolve.tex, line 666::

  sage: x = A.solve_right(b)

Sage example in ./linsolve.tex, line 678::

  sage: A = matrix(RDF, [[-1,2],[3,4]])
  sage: P, L, U = A.LU()

Sage example in ./linsolve.tex, line 723::

  sage: eps = 1e-16
  sage: y = (1-2*eps)/(1-eps)
  sage: x = (1-y)/eps
  sage: x, y
  (1.11022302462516, 1.00000000000000)

Sage example in ./linsolve.tex, line 736::

  sage: 1. + eps == 1.
  True

Sage example in ./linsolve.tex, line 758::

  sage: y = (1-2*eps)/(1-eps)
  sage: x = 2-y
  sage: x, y
  (1.00000000000000, 1.00000000000000)

Sage example in ./linsolve.tex, line 803::

  sage: A = random_matrix(RDF, 1000)
  sage: b = vector(RDF, range(1000))
  sage: c = vector(RDF, 2*list(range(500)))

Sage example in ./linsolve.tex, line 858::

  sage: m = random_matrix(RDF, 10)
  sage: A = transpose(m)*m
  sage: C = A.cholesky()

Sage example in ./linsolve.tex, line 946::

  sage: A = random_matrix(RDF,6,5)
  sage: Q, R = A.QR()

Sage example in ./linsolve.tex, line 1012::

  sage: A = matrix(RDF, [[1,3,2],[1,2,3],[0,5,2],[1,1,1]])
  sage: U, Sig, V = A.SVD()
  sage: A1 = A - U*Sig*transpose(V); A1 # abs tol 1e-9
  [ 2.220446049250313e-16                    0.0                    0.0]
  [3.3306690738754696e-16 -4.440892098500626e-16 -4.440892098500626e-16]
  [-9.298117831235686e-16 1.7763568394002505e-15 -4.440892098500626e-16]
  [ 4.440892098500626e-16 -8.881784197001252e-16 -4.440892098500626e-16]

Sage example in ./linsolve.tex, line 1123::

  sage: A = matrix(RDF, [[1,3,2],[1,4,2],[0,5,2],[1,3,2]])
  sage: b = vector(RDF, [1,2,3,4])
  sage: Z = transpose(A)*A
  sage: C = Z.cholesky()
  sage: R = transpose(A)*b
  sage: Z.solve_right(R) # abs tol 2e-13
  (-1.5000000000000135, -0.5000000000000085, 2.7500000000000213)

Sage example in ./linsolve.tex, line 1188::

  sage: A = matrix(RDF, [[1,3,2],[1,4,2],[0,5,2],[1,3,2]])
  sage: b = vector(RDF, [1,2,3,4])
  sage: Q, R = A.QR()
  sage: R1 = R[0:3,0:3]
  sage: b1 = transpose(Q)*b
  sage: c = b1[0:3]
  sage: R1.solve_right(c)  # abs tol 1e-13
  (-1.499999999999999, -0.49999999999999867, 2.749999999999997)

Sage example in ./linsolve.tex, line 1202::

  sage: Z = A.transpose()*A
  sage: Z.norm(Infinity)*(Z^-1).norm(Infinity)  # abs tol 2e-10
  1992.3750000000168

Sage example in ./linsolve.tex, line 1256::

  sage: A = matrix(RDF, [[1,3,2],[1,3,2],[0,5,2],[1,3,2]])
  sage: B = vector(RDF, [1,2,3,4])
  sage: U, Sig, V = A.SVD()
  sage: m = A.ncols()
  sage: x = vector(RDF, [0]*m)
  sage: lamb = vector(RDF, [0]*m)
  sage: for i in range(0,m):
  ....:     s = Sig[i,i]
  ....:     if s!=0.0:
  ....:         lamb[i]=U.column(i)*B/s
  sage: x = V*lamb; x  # random
  (0.2370370370370367, 0.4518518518518521, 0.3703703703703702)

Sage example in ./linsolve.tex, line 1289::

  sage: m = 3; [ Sig[i,i] for i in range(0,m) ]  # abs tol 1e-15
  [8.309316833256451, 1.3983038884881154, 0.0]

Sage example in ./linsolve.tex, line 1358::

  sage: A = matrix(RDF, [[1,2],[3,4],[5,6],[7,8]])

Sage example in ./linsolve.tex, line 1366::

  sage: set_random_seed(0) # to get reproducible values below

Sage example in ./linsolve.tex, line 1369::

  sage: th = 0.7
  sage: R = matrix(RDF, [[cos(th),sin(th)],[-sin(th),cos(th)]])
  sage: B = (A + 0.1*random_matrix(RDF,4,2)) * transpose(R)

Sage example in ./linsolve.tex, line 1374::

  sage: C = transpose(B)*A
  sage: U, Sigma, V = C.SVD()
  sage: Q = U*transpose(V)

Sage example in ./linsolve.tex, line 1381::

  sage: Q  # rel tol 1e-15
  [ 0.7612151656410957   0.648499399843978]
  [-0.6484993998439779  0.7612151656410955]
  sage: R
  [0.7648421872844885  0.644217687237691]
  [-0.644217687237691 0.7648421872844885]

Sage example in ./linsolve.tex, line 1562::

  sage: set_random_seed(0) # to get reproducible values below

Sage example in ./linsolve.tex, line 1675::

  sage: A = matrix(RDF, [[1,3,2],[1,2,3],[0,5,2]])

Sage example in ./linsolve.tex, line 1698::

  sage: A = matrix(RDF,[[1,3,2],[1,2,3],[0,5,2]])
  sage: mu = 0.56
  sage: AT = A - mu*identity_matrix(RDF,3)
  sage: X = vector(RDF,[1 for i in range(0,A.nrows())])
  sage: lam_old = 0
  sage: for i in range(1,1000): # abs tol 1e-9
  ....:     Z = AT.solve_right(X)
  ....:     X = Z/Z.norm()
  ....:     lam = X.dot_product(A*X)
  ....:     s = abs(lam - lam_old)
  ....:     print("{i} s={s} lambda={lam}".format(i=i, s=s, lam=lam))
  ....:     lam_old = lam
  ....:     if s<1.e-10:
  ....:         break
  1 s=0.56423627407 lambda=0.56423627407
  2 s=0.00371649959176 lambda=0.560519774478
  3 s=2.9833340176e-07 lambda=0.560519476145
  4 s=3.30288019157e-11 lambda=0.560519476112
  sage: X  # abs tol 1e-15
  (0.9276845629439007, 0.10329475725387141, -0.3587917847435305)

Sage example in ./linsolve.tex, line 1724::

  sage: A*X-lam*X # abs tol 1e-14
  (2.886579864025407e-15, 1.672273430841642e-15, 8.326672684688674e-15)

Sage example in ./linsolve.tex, line 1801::

  sage: m = matrix(RDF, [[1,2,3,4],[1,0,2,6],[1,8,4,-2],[1,5,-10,-20]])
  sage: Aref = transpose(m)*m
  sage: A = copy(Aref)
  sage: for i in range(0,20):
  ....:     Q, R = A.QR()
  ....:     A = R*Q

Sage example in ./linsolve.tex, line 1835::

  sage: Aref.eigenvalues()  # abs tol 1e-12
  [585.0305586200212, 92.91426499150643, 0.03226690899408103, 4.022909479477674]

Sage example in ./linsolve.tex, line 1895::

  sage: A = matrix(RDF, [[1,3,2],[1,2,3],[0,5,2]])
  sage: eigen_vals, eigen_vects = A.eigenmatrix_right()
  sage: eigen_vals  # abs tol 1e-12
  [   6.39294791648918                 0.0                 0.0]
  [                0.0   0.560519476111939                 0.0]
  [                0.0                 0.0 -1.9534673926011215]
  sage: eigen_vects # abs tol 2e-15
  [ 0.5424840601106511  0.9276845629439008 0.09834254667424457]
  [ 0.5544692861094349 0.10329475725386986  -0.617227053099068]
  [ 0.6310902116870117 -0.3587917847435306   0.780614827194734]

Sage example in ./linsolve.tex, line 1939::

  sage: def pol2companion(p):
  ....:     n = len(p)
  ....:     m = matrix(RDF,n)
  ....:     for i in range(1,n):
  ....:         m[i,i-1]=1
  ....:     m.set_column(n-1,-p)
  ....:     return m

Sage example in ./linsolve.tex, line 1965::

  sage: q = vector(RDF,[1,-1,2,3,5,-1,10,11])
  sage: comp = pol2companion(q); comp
  [  0.0   0.0   0.0   0.0   0.0   0.0   0.0  -1.0]
  [  1.0   0.0   0.0   0.0   0.0   0.0   0.0   1.0]
  [  0.0   1.0   0.0   0.0   0.0   0.0   0.0  -2.0]
  [  0.0   0.0   1.0   0.0   0.0   0.0   0.0  -3.0]
  [  0.0   0.0   0.0   1.0   0.0   0.0   0.0  -5.0]
  [  0.0   0.0   0.0   0.0   1.0   0.0   0.0   1.0]
  [  0.0   0.0   0.0   0.0   0.0   1.0   0.0 -10.0]
  [  0.0   0.0   0.0   0.0   0.0   0.0   1.0 -11.0]
  sage: roots = comp.eigenvalues(); roots  # abs tol 1e-12
  [0.3475215101190289 + 0.5665505533984981*I, 0.3475215101190289 - 0.5665505533984981*I,
   0.34502377696179265 + 0.43990870238588275*I, 0.34502377696179265 - 0.43990870238588275*I,
   -0.5172576143252197 + 0.5129582067889322*I, -0.5172576143252197 - 0.5129582067889322*I,
   -1.3669971645459291, -9.983578180965276]

Sage example in ./linsolve.tex, line 2121::

  sage: reset()

Sage example in ./linsolve.tex, line 2124::

  sage: def eval(P,x):
  ....:     if len(P) == 0:
  ....:         return 0
  ....:     else:
  ....:         return P[0]+x*eval(P[1:],x)

Sage example in ./linsolve.tex, line 2133::

  sage: def pscal(P,Q,lx):
  ....:     return float(sum(eval(P,s)*eval(Q,s) for s in lx))

Sage example in ./linsolve.tex, line 2139::

  sage: def padd(P,a,Q):
  ....:     for i in range(0,len(Q)):
  ....:         P[i] += a*Q[i]

Sage example in ./linsolve.tex, line 2149::

  sage: class BadParamsforOrthop(Exception):
  ....:     def __init__(self, degreeplusone, npoints):
  ....:         self.deg = degreeplusone - 1
  ....:         self.np = npoints
  ....:     def __str__(self):
  ....:         return "degree: " + str(self.deg) + \
  ....:                " nb. points: " + repr(self.np)

Sage example in ./linsolve.tex, line 2160::

  sage: def orthopoly(n,x):
  ....:     if n > len(x):
  ....:         raise BadParamsforOrthop(n, len(x))
  ....:     orth = [[1./sqrt(float(len(x)))]]
  ....:     for p in range(1,n):
  ....:         nextp = copy(orth[p-1])
  ....:         nextp.insert(0,0)
  ....:         s = []
  ....:         for i in range(p-1,max(p-3,-1),-1):
  ....:             s.append(pscal(nextp, orth[i], x))
  ....:         j = 0
  ....:         for i in range(p-1,max(p-3,-1),-1):
  ....:             padd(nextp, -s[j], orth[i])
  ....:             j += 1
  ....:         norm = sqrt(pscal(nextp, nextp, x))
  ....:         nextpn = [nextp[i]/norm for i in range(len(nextp))]
  ....:         orth.append(nextpn)
  ....:     return orth

Sage example in ./linsolve.tex, line 2209::

  sage: set_random_seed(3) # to get reproducible values below

Sage example in ./linsolve.tex, line 2212::

  sage: L = 40
  sage: X = [100*float(i)/L for i in range(40)]
  sage: Y = [float(1/(1+25*X[i]^2)+0.25*random()) for i in range(40)]
  sage: n = 15; orth = orthopoly(n, X)

Sage example in ./linsolve.tex, line 2222::

  sage: coeff = [sum(Y[j]*eval(orth[i],X[j]) for j in
  ....:         range(0,len(X))) for i in range(0,n)]

Sage example in ./linsolve.tex, line 2230::

  sage: polmin = [0 for i in range(0,n)]
  sage: for i in range(0,n):
  ....:     padd(polmin, coeff[i], orth[i])
  sage: p = lambda x: eval(polmin, x)
  sage: plot(p(x), x, 0, X[len(X)-1])
  Graphics object consisting of 1 graphics primitive

Sage example in ./linsolve.tex, line 2609::

  sage: from scipy.sparse.linalg import factorized
  sage: from scipy.sparse import lil_matrix
  sage: from numpy import array
  sage: n = 200
  sage: n2 = n*n
  sage: A = lil_matrix((n2, n2))
  sage: h2 = 1./float((n+1)^2)
  sage: for i in range(0,n2):
  ....:    A[i,i]=4*h2+1.
  ....:    if i+1<n2: A[i,int(i+1)]=-h2
  ....:    if i>0:    A[i,int(i-1)]=-h2
  ....:    if i+n<n2: A[i,int(i+n)]=-h2
  ....:    if i-n>=0: A[i,int(i-n)]=-h2
  sage: Acsc = A.tocsc()
  sage: b = array([1 for i in range(0,n2)])
  sage: solve = factorized(Acsc) # LU factorization
  sage: S = solve(b)             # resolution

Sage example in ./linsolve.tex, line 2784::

  sage: from numpy.random import seed
  sage: seed(0) # to get reproducible values below

Sage example in ./linsolve.tex, line 2828::

  sage: from scipy import sparse
  sage: from numpy.linalg import *
  sage: from numpy import array
  sage: from numpy.random import rand
  sage: def power(A,x,N):             # power iteration
  ....:     for i in range(N):
  ....:         y = A*x
  ....:         z = y/norm(y)
  ....:         lam = sum(x*y)
  ....:         s = norm(x-z)
  ....:         print("{i} s={s} lambda={lam}".format(i=i, s=s, lam=lam))
  ....:         if s < 1e-3:
  ....:             break
  ....:         x = z
  ....:     return x
  sage: n = 1000
  sage: m = 5
  sage: # build a stochastic matrix of size n
  sage: # with m nonzero coefficients per row
  sage: A1 = sparse.lil_matrix((n, n))
  sage: for i in range(0,n):
  ....:     for j in range(0,m):
  ....:         l = int(n*rand())
  ....:         A1[l,i] = rand()
  sage: for i in range(0,n):
  ....:     s = sum(A1[i,0:n])
  ....:     A1[i,0:n] /= s
  sage: At = A1.transpose().tocsc()
  sage: x = array([rand() for i in range(0,n)])
  sage: # compute the dominant eigenvalue
  sage: # and the associated eigenvector
  sage: y = power(At, x, 5)  # rel tol 1e-10
  0 s=17.0241218112 lambda=235.567796432
  1 s=0.39337173784 lambda=0.908668201953
  2 s=0.230865716856 lambda=0.967356896036
  3 s=0.134156683993 lambda=0.986660315554
  4 s=0.0789423487458 lambda=0.995424635219
"""



# sage_setup: distribution = sagemath-repl
r"""
Rich Output for the Browser
"""

import re

from sage.repl.rich_output.output_basic import OutputBase
from sage.repl.rich_output.buffer import OutputBuffer

# regex to match "<html>\[...\]</html>" or "<html>\(...\)</html>"
latex_re = re.compile(r'<html>(?P<mathstart>\\\[|\\\()(?P<latex>.*)(?P<mathend>\\\]|\\\))</html>',
                      flags=re.DOTALL)


class OutputHtml(OutputBase):

    def __init__(self, html):
        """
        HTML Output.

        INPUT:

        - ``html`` --
          :class:`~sage.repl.rich_output.buffer.OutputBuffer`. Alternatively, a
          string (bytes) can be passed directly which will then be converted
          into an :class:`~sage.repl.rich_output.buffer.OutputBuffer`. String
          containing the html fragment code. Excludes the surrounding
          ``<body>`` and ``<html>`` tag.

        EXAMPLES::

            sage: from sage.repl.rich_output.output_catalog import OutputHtml
            sage: OutputHtml('<div>Foo<b>B</b>ar</div>')
            OutputHtml container
        """
        self.html = OutputBuffer(html)

        # if the html is a simple wrapper of latex for mathjax rendering, then
        # the latex string is saved for possible latex output such as Jupyter's
        # pdf export of a notebook
        m = latex_re.match(html)
        if m:
            mathjax_string = m.group('latex')
            latex_string = mathjax_string.replace('&lt;', '<')
            if m.group('mathstart') == r'\[' and m.group('mathend') == r'\]':
                self.latex = OutputBuffer('$$' + latex_string + '$$')
            else:
                self.latex = OutputBuffer('$' + latex_string + '$')
        else:
            self.latex = None

    @classmethod
    def example(cls):
        r"""
        Construct a sample Html output container.

        This static method is meant for doctests, so they can easily
        construct an example.

        OUTPUT: an instance of :class:`OutputHtml`

        EXAMPLES::

            sage: from sage.repl.rich_output.output_catalog import OutputHtml
            sage: OutputHtml.example()
            OutputHtml container
            sage: OutputHtml.example().html.get_str()
            '<div>Hello World!</div>'
        """
        return cls('<div>Hello World!</div>')

    def print_to_stdout(self):
        r"""
        Write the data to stdout.

        This is just a convenience method to help with debugging.

        EXAMPLES::

            sage: from sage.repl.rich_output.output_catalog import OutputHtml
            sage: rich_output = OutputHtml.example()
            sage: rich_output.print_to_stdout()
            <div>Hello World!</div>
        """
        print(self.html.get_unicode())

    def with_html_tag(self):
        r"""
        Return the HTML code surrounded by ``<html>`` tag.

        This is just a convenience method.

        EXAMPLES::

            sage: from sage.repl.rich_output.output_catalog import OutputHtml
            sage: rich_output = OutputHtml.example()
            sage: rich_output.print_to_stdout()
            <div>Hello World!</div>
            sage: rich_output.with_html_tag()
            '<html><div>Hello World!</div></html>'
        """
        return '<html>{0}</html>'.format(self.html.get_unicode())



# sage_setup: distribution = sagemath-repl

from sage.misc.all__sagemath_objects import *

from sage.misc.sage_eval import sage_eval, sageobj

from sage.misc.sage_input import sage_input

from sage.misc.banner import version

lazy_import('sage.misc.banner', 'banner', deprecation=34259)

lazy_import('sage.misc.sagedoc', ['browse_sage_doc',
                                  'search_src', 'search_def', 'search_doc',
                                  'tutorial', 'reference', 'manual', 'developer',
                                  'constructions', 'help'])

lazy_import('pydoc', 'help', 'python_help')

from sage.misc.explain_pickle import (explain_pickle, unpickle_newobj, unpickle_build,
                                      unpickle_instantiate, unpickle_persistent,
                                      unpickle_extension, unpickle_appends)

lazy_import('sage.misc.trace', 'trace', deprecation=34259)

lazy_import('sage.misc.profiler', 'Profiler', deprecation=34259)

from sage.misc.dev_tools import import_statements

lazy_import('sage.misc.dev_tools', 'runsnake', deprecation=34259)

from sage.misc.edit_module import edit

lazy_import('sage.misc.edit_module', 'set_edit_template', deprecation=34259)

lazy_import('sage.misc.pager', 'pager')


lazy_import("sage.misc.cython", "cython_lambda")
lazy_import("sage.misc.cython", "cython_compile", "cython")
lazy_import('sage.misc.inline_fortran', 'fortran')

lazy_import('sage.misc.package', ('installed_packages', 'is_package_installed',
                                  'package_versions'),
            deprecation=34259)
lazy_import('sage.misc.benchmark', 'benchmark', deprecation=34259)
